#!/usr/local/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test UserJob module."""

from __future__ import absolute_import
import unittest
from mock import patch, MagicMock as Mock

from DIRAC import S_OK, S_ERROR
from ILCDIRAC.Interfaces.API.NewInterface.UserJob import UserJob

from ILCDIRAC.Interfaces.API.NewInterface.Applications import Fcc

from Tests.Utilities.GeneralUtils import assertDiracFailsWith, \
    assertDiracSucceeds, assertDiracSucceedsWith_equals, assertMockCalls, \
    assertEqualsImproved
from six.moves import range

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Interfaces.API.NewInterface.UserJob'
MIXIN_MODULE = 'ILCDIRAC.Interfaces.Utilities.SplitMixin'
MIXIN_CLASS = 'ILCDIRAC.Interfaces.Utilities.SplitMixin.SplitMixin'

# pylint: disable=protected-access, too-many-public-methods


class UserJobTestCase(unittest.TestCase):
  """Base class for the UserJob test cases."""

  def setUp(self):
    """Set up the objects."""
    self.log_mock = Mock(name="SubMock")
    with patch('%s.getProxyInfo' % MODULE_NAME, new=Mock(return_value=None)):
      self.ujo = UserJob()

  def test_submit_noproxy(self):
    self.ujo.proxyinfo = S_ERROR()
    assertDiracFailsWith(self.ujo.submit(),
                          "Not allowed to submit a job, you need a ['ilc_user', 'calice_user', 'fcc_user'] proxy", self)

  def test_submit_wrongproxygroup(self):
    self.ujo.proxyinfo = S_OK({'group': 'my_test_group.notInallowed_list'})
    assertDiracFailsWith(self.ujo.submit(),
                          "Not allowed to submit job, you need a ['ilc_user', 'calice_user', 'fcc_user'] proxy", self)

  def test_submit_noproxygroup(self):
    self.ujo.proxyinfo = S_OK({'some_key': 'Value', True: 1, False: [], 135: {}})
    assertDiracFailsWith(self.ujo.submit(), 'Could not determine group, you do not have the right proxy', self)

  def test_submit_addtoworkflow_fails(self):
    self.ujo.proxyinfo = S_OK({'group': 'ilc_user'})
    with patch('%s.UserJob._addToWorkflow' % MODULE_NAME, new=Mock(return_value=S_ERROR('workflow_testadd_error'))):
      assertDiracFailsWith(self.ujo.submit(), 'workflow_testadd_error', self)

  def test_submit_addtoworkflow_fails_2(self):
    self.ujo.proxyinfo = S_OK({'group': 'calice_user'})
    with patch('%s.UserJob._addToWorkflow' % MODULE_NAME, new=Mock(return_value=S_ERROR('err_workflow_testadd'))):
      assertDiracFailsWith(self.ujo.submit(), 'err_workflow_testadd', self)

  def test_submit_createnew_dirac_instance(self):
    ilc_mock = Mock()
    ilc_mock().submitJob.return_value = S_OK('test_submission_successful')
    self.ujo.proxyinfo = S_OK({'group': 'ilc_user'})
    with patch('%s.UserJob._addToWorkflow' % MODULE_NAME, new=Mock(return_value=S_OK())), \
         patch('%s.DiracILC' % MODULE_NAME, new=ilc_mock):
      assertDiracSucceedsWith_equals(self.ujo.submit(), 'test_submission_successful', self)
      ilc_mock().submitJob.assert_called_once_with(self.ujo, 'wms')
      assert self.ujo.oktosubmit

  def test_submit_existing_dirac_instance(self):
    """Test submit with dirac instance."""
    ilc_mock = Mock()
    ilc_mock.submitJob.return_value = S_OK('test_submission_successful')
    self.ujo.proxyinfo = S_OK({'group': 'ilc_user'})
    with patch('%s.UserJob._addToWorkflow' % MODULE_NAME, new=Mock(return_value=S_OK())):
      assertDiracSucceedsWith_equals(self.ujo.submit(diracinstance=ilc_mock), 'test_submission_successful', self)
      ilc_mock.submitJob.assert_called_once_with(self.ujo, 'wms')
      assert self.ujo.oktosubmit

  def test_setinputdata_failed(self):
    assertDiracFailsWith(self.ujo.setInputData({'/mylfn1': True, '/mylfn2': False}),
                          'expected lfn string or list of lfns for input data', self)

  def test_setinputdata(self):
    """Test setting input data."""
    assertDiracSucceeds(self.ujo.setInputData(['LFN:/mylfn1', 'LFN:/mylfn2']), self)
    self.assertEqual(self.ujo.workflow.parameters.find('InputData').getValue(), '/mylfn1;/mylfn2')
    assertDiracSucceeds(self.ujo.setInputData('/mylfn1'), self)
    self.assertEqual(self.ujo.workflow.parameters.find('InputData').getValue(), '/mylfn1')

  def test_inputsandbox(self):
    self.ujo.inputsandbox = Mock()
    assertDiracSucceeds(self.ujo.setInputSandbox('LFN:/ilc/user/u/username/libraries.tar.gz'), self)
    self.ujo.inputsandbox.extend.assert_called_once_with(['LFN:/ilc/user/u/username/libraries.tar.gz'])

  def test_inputsandbox_dictpassed(self):
    assertDiracFailsWith(self.ujo.setInputSandbox({'/some/file': True, '/my/dict': True}),
                          'File passed must be either single file or list of files', self)

  def test_setOutputData(self):
    """Test setting output data."""
    assertDiracSucceeds(self.ujo.setOutputData(['/myFile1', '/myFile2']), self)
    self.assertEqual(self.ujo.workflow.parameters.find('UserOutputData').getValue(), '/myFile1;/myFile2')

    assertDiracSucceeds(self.ujo.setOutputData('/myFile2'), self)
    self.assertEqual(self.ujo.workflow.parameters.find('UserOutputData').getValue(), '/myFile2')

    assertDiracSucceeds(self.ujo.setOutputData('/myFile2', OutputSE="MY-SE"), self)
    self.assertEqual(self.ujo.workflow.parameters.find('UserOutputData').getValue(), '/myFile2')
    self.assertEqual(self.ujo.workflow.parameters.find('UserOutputSE').getValue(), 'MY-SE')

    assertDiracSucceeds(self.ujo.setOutputData('/myFile2', OutputSE=["MY-SE", 'YOUR-SE']), self)
    self.assertEqual(self.ujo.workflow.parameters.find('UserOutputData').getValue(), '/myFile2')
    self.assertEqual(self.ujo.workflow.parameters.find('UserOutputSE').getValue(), 'MY-SE;YOUR-SE')

  def test_setoutputdata_dictpassed(self):
    assertDiracFailsWith(self.ujo.setOutputData({'/mydict': True}),
                          'Expected file name string or list of file names for output data', self)

  def test_setoutputdata_nolistse(self):
    wf_mock = Mock()
    self.ujo.workflow = wf_mock
    with patch('%s.UserJob._addParameter' % MODULE_NAME, new=Mock()) as addparam_mock:
      assertDiracFailsWith(self.ujo.setOutputData(['mylfn1', 'other_lfn', 'last___lfn'],
                                                  OutputSE={'mydict': True}),
                           'Expected string or list for OutputSE', self)
      addparam_mock.assert_called_once_with(wf_mock, 'UserOutputData', 'JDL',
                                            'mylfn1;other_lfn;last___lfn', 'List of output data files')

  def test_setoutputdata_outputpath_nostring(self):
    wf_mock = Mock()
    self.ujo.workflow = wf_mock
    with patch('%s.UserJob._addParameter' % MODULE_NAME, new=Mock()) as addparam_mock:
      assertDiracFailsWith(self.ujo.setOutputData(['mylfn1', 'other_lfn', 'last___lfn'],
                                                    OutputPath={'mydict': True}),
                            'Expected string for OutputPath', self)
      addparam_mock.assert_called_once_with(wf_mock, 'UserOutputData', 'JDL',
                                             'mylfn1;other_lfn;last___lfn', 'List of output data files')

  def test_setoutputdata_invalid_outputpath_1(self):
    wf_mock = Mock()
    self.ujo.workflow = wf_mock
    with patch('%s.UserJob._addParameter' % MODULE_NAME, new=Mock()) as addparam_mock:
      assertDiracFailsWith(self.ujo.setOutputData(['mylfn1', 'other_lfn', 'last___lfn'],
                                                    OutputPath='//ilc/user/somedir/output.xml'),
                            'Output path contains /ilc/user/ which is not what you want', self)
      addparam_mock.assert_called_once_with(wf_mock, 'UserOutputData', 'JDL',
                                             'mylfn1;other_lfn;last___lfn', 'List of output data files')

  def test_setoutputdata_invalid_outputpath_2(self):
    wf_mock = Mock()
    self.ujo.workflow = wf_mock
    with patch('%s.UserJob._addParameter' % MODULE_NAME, new=Mock()) as addparam_mock:
      assertDiracFailsWith(self.ujo.setOutputData(['mylfn1', 'other_lfn', 'last___lfn'],
                                                    OutputPath='//some/dir/ilc/user/somedir/output.xml'),
                            'Output path contains /ilc/user/ which is not what you want', self)
      addparam_mock.assert_called_once_with(wf_mock, 'UserOutputData', 'JDL',
                                             'mylfn1;other_lfn;last___lfn', 'List of output data files')

  def test_setoutputdata(self):
    wf_mock = Mock()
    self.ujo.workflow = wf_mock
    with patch('%s.UserJob._addParameter' % MODULE_NAME, new=Mock()) as addparam_mock:
      assertDiracSucceeds(self.ujo.setOutputData(
          ['mylfn1', 'other_lfn', 'last___lfn'], OutputPath='//some/dir/somedir/output.xml'), self)
      assertMockCalls(addparam_mock, [
          (wf_mock, 'UserOutputData', 'JDL', 'mylfn1;other_lfn;last___lfn', 'List of output data files'),
          (wf_mock, 'UserOutputPath', 'JDL', 'some/dir/somedir/output.xml', 'User specified Output Path')],
          self)

  def test_setoutputsandbox(self):
    wf_mock = Mock()
    self.ujo.workflow = wf_mock
    with patch('%s.UserJob._addParameter' % MODULE_NAME, new=Mock()) as addparam_mock:
      assertDiracSucceeds(self.ujo.setOutputSandbox('/my/dir/myfile.txt'), self)
      addparam_mock.assert_called_once_with(wf_mock, 'OutputSandbox', 'JDL',
                                             '/my/dir/myfile.txt', 'Output sandbox file')

  def test_setoutputsandbox_successes(self):
    """Test setting output sandbox."""
    assertDiracSucceeds(self.ujo.setOutputSandbox(['myfile.txt', 'myfile.doc']), self)
    self.assertEqual(self.ujo.workflow.parameters.find('OutputSandbox').getValue(), 'myfile.txt;myfile.doc')

  def test_setoutputsandbox_dictpassed(self):
    wf_mock = Mock()
    self.ujo.workflow = wf_mock
    with patch('%s.UserJob._addParameter' % MODULE_NAME, new=Mock()) as addparam_mock:
      assertDiracFailsWith(self.ujo.setOutputSandbox({'mydict': True}),
                            'Expected file string or list of files for output sandbox contents', self)
      self.assertFalse(addparam_mock.called)

  def test_configs(self):
    """Test setting different config packages."""
    assertDiracSucceeds(self.ujo.setILDConfig('123.4'), self)
    self.assertEqual(self.ujo.workflow.parameters.find('ILDConfigPackage').getValue(), 'ILDConfig123.4')

    assertDiracSucceeds(self.ujo.setCLICConfig('567.8'), self)
    self.assertEqual(self.ujo.workflow.parameters.find('ClicConfigPackage').getValue(), 'ClicConfig567.8')

    self.assertIn('ildconfig', self.ujo.workflow.parameters.find('SoftwarePackages').getValue())
    self.assertIn('clicconfig', self.ujo.workflow.parameters.find('SoftwarePackages').getValue())

  def test_submit_split(self):
    """Test submitting with automatic splitting."""
    self.ujo._splittingOption = True
    self.ujo._split = Mock(return_value=S_OK())
    self.ujo.proxyinfo = S_OK({'group': 'ilc_user'})
    ilc_mock = Mock()
    ilc_mock.submitJob.return_value = S_OK('test_submission_successful')
    assertDiracSucceeds(self.ujo.submit(diracinstance=ilc_mock), self)
    ilc_mock.submitJob.assert_called_once_with(self.ujo, 'wms')

    self.ujo._splittingOption = True
    self.ujo._split = Mock(return_value=S_ERROR("Splitting InValid"))
    assertDiracFailsWith(self.ujo.submit(), "Splitting InValid", self)

  @patch("%s._checkSplitConsistency" % MIXIN_CLASS, new=Mock(return_value=S_OK()))
  def test_split_bydata(self):
    """Test splitting by data."""
    self.ujo._eventsPerJob = "1"
    self.ujo._numberOfJobs = "1"
    self.ujo._splittingOption = "byData"
    self.ujo._switch['byData'] = Mock(return_value=[("InputData", ["/ilc/user/u/username/data1"], True)])
    with patch("%s.UserJob.setParameterSequence" % MODULE_NAME) as mock_parametric, \
         patch('%s.LOG' % MIXIN_MODULE, new=self.log_mock):
      info_message = "Job splitting successful"
      assertDiracSucceeds(self.ujo._split(), self)
      self.log_mock.notice.assert_called_with(info_message)
      mock_parametric.assert_any_call("InputData", ["/ilc/user/u/username/data1"], True)

  @patch("%s.toInt" % MIXIN_MODULE, new=Mock(return_value=1))
  @patch("%s.UserJob._checkSplitConsistency" % MODULE_NAME, new=Mock(return_value=S_OK()))
  def test_split_byevents(self):
    """Test splitting by events."""
    self.ujo._splittingOption = "byEvents"
    self.ujo._switch['byEvents'] = Mock(return_value=[('NumberOfEvents', [1, 2], 'NbOfEvts')])
    with patch("%s.UserJob.setParameterSequence" % MODULE_NAME) as mock_parametric, \
         patch('%s.LOG' % MIXIN_MODULE, new=self.log_mock):
      info_message = "Job splitting successful"
      assertDiracSucceeds(self.ujo._split(), self)
      self.log_mock.notice.assert_called_with(info_message)
      mock_parametric.assert_any_call('NumberOfEvents', [1, 2], 'NbOfEvts')

  @patch("%s.toInt" % MIXIN_MODULE, new=Mock(return_value=1))
  @patch("%s.UserJob._checkSplitConsistency" % MODULE_NAME, new=Mock(return_value=S_OK()))
  def test_split_atomicsubmission(self):
    """Test splitting atomic."""
    self.ujo._splittingOption = None
    info_message = "Job splitting successful"
    with patch('%s.LOG' % MIXIN_MODULE, new=self.log_mock):
      assertDiracSucceeds(self.ujo._split(), self)
    self.log_mock.notice.assert_called_with(info_message)

  @patch("%s.toInt" % MIXIN_MODULE, new=Mock(return_value=False))
  def test_split_inputparameters_failed(self):
    """Test splitting input parameters with failure."""
    assertDiracFailsWith(self.ujo._split(), "Splitting: Invalid values for splitting", self)

  @patch("%s.toInt" % MIXIN_MODULE, new=Mock(return_value=1))
  @patch("%s._checkSplitConsistency" % MIXIN_CLASS, new=Mock(return_value=S_ERROR('failed')))
  def test_split_checkSplitConsistency_failed(self):
    """Test splitting check consistency with failure."""
    assertDiracFailsWith(self.ujo._split(), 'failed', self)

  @patch("%s.toInt" % MIXIN_MODULE, new=Mock(return_value=1))
  @patch("%s._checkSplitConsistency" % MIXIN_CLASS, new=Mock(return_value=S_OK()))
  def test_split_sequencer_fails(self):
    """Test splitting when the sequencer fails."""
    self.ujo._splittingOption = "bySequence"
    self.ujo._switch['bySequence'] = Mock(return_value=[])
    self.ujo.setParameterSequence = Mock()
    self.ujo._split()
    self.ujo.setParameterSequence.assert_not_called()

  def test_checkSplitconsistency(self):
    """Test splitting consistency check."""
    app1 = Fcc()
    app2 = Fcc()
    self.ujo.applicationlist = [app1, app2]
    self.ujo._switch = {"byEvents": lambda x: x}
    self.ujo._splittingOption = "byEvents"
    self.assertTrue(self.ujo._checkSplitConsistency())

  def test_checkjobconsistency_bad_split_parameter(self):
    """Test splitting consistency check with bad split parameters."""
    app1 = Fcc()
    app2 = Fcc()
    self.ujo.applicationlist = [app1, app2]
    self.ujo._switch = {"byEvents": lambda x: x}
    self.ujo._splittingOption = "byHand"
    self.assertFalse(self.ujo._checkSplitConsistency()['OK'])
    self.assertIn('_checkSplitConsistency', self.ujo.errorDict)

  def test_checkjobconsistency_no_same_events(self):
    app1 = Fcc()
    app2 = Fcc()
    app1.numberOfEvents = 1
    app2.numberOfEvents = 2
    self.ujo.applicationlist = [app1, app2]
    self.ujo._switch = {"byEvents": lambda x: x}
    self.ujo._splittingOption = "byEvents"
    with patch('%s.LOG' % MIXIN_MODULE, new=self.log_mock):
      resCheck = self.ujo._checkSplitConsistency()
    self.assertFalse(resCheck['OK'])
    self.assertIn("have the same number", resCheck['Message'])

  def test_checkjobconsistency_negative_events(self):
    app1 = Fcc()
    app2 = Fcc()
    app1.numberOfEvents = app2.numberOfEvents = -1
    self.ujo.applicationlist = [app1, app2]
    self.ujo._switch = ["byEvents"]
    self.ujo._splittingOption = "byEvents"
    self.assertTrue(self.ujo._checkSplitConsistency())

  def test_splitbydata(self):
    self.ujo._data = ['data1', 'data2']
    app1 = Fcc()
    app2 = Fcc()
    self.ujo.applicationlist = [app1, app2]
    assertEqualsImproved(self.ujo._splitByData(), [("InputData", [['data1'], ['data2']], 'ParametricInputData')], self)

  def test_splitbydata_no_data(self):
    """Test splitting without data."""
    self.ujo._data = None
    self.assertFalse(self.ujo._splitByData())
    self.assertIn('_splitByData', self.ujo.errorDict)

  def test_splitbydata_incorrectparameter(self):
    """Test splitting with data."""
    self.ujo._data = ["/path/to/data1", "/path/to/data2"]
    self.ujo._numberOfFilesPerJob = 3
    self.assertFalse(self.ujo._splitByData())
    self.assertIn('_splitByData', self.ujo.errorDict)

  def test_splitbyevents_1st_case(self):
    """Test splitting by events."""
    app1 = Fcc()
    app2 = Fcc()
    self.ujo.applicationlist = [app1, app2]
    self.ujo._eventsPerJob = 2
    self.ujo._numberOfJobs = 3
    map_event_job = [2, 2, 2]
    assertEqualsImproved(self.ujo._splitByEvents(), [('NumberOfEvents', map_event_job, 'NbOfEvts')], self)

  def test_splitbyevents_2nd_case(self):
    app1 = Fcc()
    app2 = Fcc()
    self.ujo.applicationlist = [app1, app2]
    self.ujo._eventsPerJob = 3
    self.ujo._totalNumberOfEvents = 5
    map_event_job = [3, 2]
    assertEqualsImproved(self.ujo._splitByEvents(), [('NumberOfEvents', map_event_job, 'NbOfEvts')], self)

  def test_splitbyevents_2nd_case_failed(self):
    app1 = Fcc()
    app2 = Fcc()
    self.ujo.applicationlist = [app1, app2]
    self.ujo._eventsPerJob = 3
    self.ujo._totalNumberOfEvents = 2
    self.assertFalse(self.ujo._splitByEvents())

  def test_splitbyevents_3rd_case(self):
    """Test splitting by events case 3."""
    app1 = Fcc()
    app2 = Fcc()
    self.ujo.applicationlist = [app1, app2]
    self.ujo._numberOfJobs = 2
    self.ujo._totalNumberOfEvents = 2
    map_event_job = [1, 1]
    assertEqualsImproved(self.ujo._splitByEvents(), [('NumberOfEvents', map_event_job, 'NbOfEvts')], self)

    self.ujo._numberOfJobs = 3
    self.ujo._totalNumberOfEvents = 5
    map_event_job = [2, 2, 1]
    assertEqualsImproved(self.ujo._splitByEvents(), [('NumberOfEvents', map_event_job, 'NbOfEvts')], self)

  def test_splitbyevents_3rd_case_failed(self):
    """Test splitting by events case 3 fails."""
    app1 = Fcc()
    app2 = Fcc()
    self.ujo.applicationlist = [app1, app2]
    self.ujo._numberOfJobs = 2
    self.ujo._totalNumberOfEvents = None
    self.assertFalse(self.ujo._splitByEvents())

  def test_setsplitevents(self):
    """Test splitting set split events."""
    self.ujo.setSplitEvents(42, 42, 126)
    assertEqualsImproved(self.ujo._totalNumberOfEvents, 126, self)
    assertEqualsImproved(self.ujo._eventsPerJob, 42, self)
    assertEqualsImproved(self.ujo._numberOfJobs, 42, self)
    assertEqualsImproved(self.ujo._splittingOption, "byEvents", self)

  def test_setsplitInputdata(self):
    """Test set split input data."""
    input_data = ["/path/to/data1", "/path/to/data2"]
    self.ujo.setSplitInputData(input_data)
    for data in input_data:
      self.assertIn(data, self.ujo._data)
    assertEqualsImproved(self.ujo._splittingOption, "byData", self)

  def test_setSplitFiles(self):
    """Test set split files over jobs."""
    self.ujo.setSplitFilesAcrossJobs('myLFN', 20, 20)
    self.assertEqual(self.ujo._data, ['myLFN'])
    self.assertEqual(self.ujo._eventsPerFile, 20)
    self.assertEqual(self.ujo._eventsPerJob, 20)

  def test_splitBySkip(self):
    """Test set split with skip."""
    self.ujo._eventsPerFile = 13
    self.ujo._eventsPerJob = 5
    self.ujo._data = ['lfn_%d' % d for d in [1, 2]]
    result = self.ujo._splitBySkip()
    self.assertEqual([('InputData', ['lfn_1', 'lfn_1', 'lfn_1', 'lfn_2', 'lfn_2', 'lfn_2'], 'InputData'),
                      ('startFrom', [0, 5, 10, 0, 5, 10], 'startFrom'),
                      ('NumberOfEvents', [5, 5, 3, 5, 5, 3], 'NbOfEvts')],
                     result)

    self.ujo._eventsPerFile = 15
    self.ujo._eventsPerJob = 5
    self.ujo._data = ['lfn_%d' % d for d in [1, 2]]
    result = self.ujo._splitBySkip()
    self.assertEqual([('InputData', ['lfn_1', 'lfn_1', 'lfn_1', 'lfn_2', 'lfn_2', 'lfn_2'], 'InputData'),
                      ('startFrom', [0, 5, 10, 0, 5, 10], 'startFrom'),
                      ('NumberOfEvents', [5, 5, 5, 5, 5, 5], 'NbOfEvts')],
                     result)

  def test_setSplittingStartIndex(self):
    """Test setting start index."""
    res = self.ujo.setSplittingStartIndex(111)
    self.assertTrue(res['OK'])
    self.assertEqual(self.ujo._startJobIndex, 111)

    self.ujo._startJobIndex = 0
    res = self.ujo.setSplittingStartIndex(-111)
    self.assertFalse(res['OK'])
    self.assertIn('setSplittingStartIndex', self.ujo.errorDict)
    self.assertEqual(self.ujo._startJobIndex, 0)

  def test_doNotAlter(self):
    """Test setting not altering the output."""
    self.ujo.setSplitDoNotAlterOutputFilename()
    self.assertIsNotNone(self.ujo.workflow.parameters.find('DoNotAlterOutputData'))
    self.assertEqual(self.ujo.workflow.parameters.find('DoNotAlterOutputData').getValue(), "True")

    self.ujo.setSplitDoNotAlterOutputFilename(False)
    self.assertIsNotNone(self.ujo.workflow.parameters.find('DoNotAlterOutputData'))
    self.assertEqual(self.ujo.workflow.parameters.find('DoNotAlterOutputData').getValue(), "False")

  def test_setSplitJobIndexList(self):
    """Test the setSplitJobIndexList function."""
    res = self.ujo.setSplitJobIndexList(list(range(0, 7, 3)))
    self.assertTrue(res['OK'])
    self.assertEqual([0, 3, 6], self.ujo._jobIndexList)

  def test_setSplitJobIndexListSet(self):
    res = self.ujo.setSplitJobIndexList(set(range(1, 7, 3)))
    self.assertTrue(res['OK'])
    self.assertIn('_checkArgs', self.ujo.errorDict)



"""Extract mean direction corrections ratio."""

from __future__ import print_function
from __future__ import absolute_import
import sys
from Helper_Functions import find_between


Calibration_File_And_Path = sys.argv[1]
Energy_To_Calibrate = sys.argv[2]

Ring_Mean_Dir_Corr = ''
EndCap_Mean_Dir_Corr = ''

with open(Calibration_File_And_Path, 'r') as f:
    searchlines = f.readlines()
    for line in searchlines:
        if 'Mean Direction Correction HCalEndCap:' in line:
            EndCap_Mean_Dir_Corr = float(find_between(line, ' : ', ' :'))
        if 'Mean Direction Correction HCalOther:' in line:
            Ring_Mean_Dir_Corr = float(find_between(line, ' : ', ' :'))

Calibration_Text = '_____________________________________________________________________________________' + '\n'

Calibration_Text += 'HCal_Direction_Corrections_Extract.py finding mean direction corrections ratio ' + '\n'
Calibration_Text += '(Ring/Other to EndCap) for HCal.' + '\n'

Calibration_Text += 'For KaonL events with energy                       :' + str(Energy_To_Calibrate) + ' /GeV \n\n'

Calibration_Text += 'Ring Mean Direction Correction                     :' + str(Ring_Mean_Dir_Corr) + ' \n\n'

Calibration_Text += 'EndCap Mean Direction Correction                   :' + str(EndCap_Mean_Dir_Corr) + ' \n\n'

with open(Calibration_File_And_Path, 'a') as myfile:
    myfile.write(Calibration_Text)

print(str(EndCap_Mean_Dir_Corr / Ring_Mean_Dir_Corr))



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Created on Feb 8, 2012.

:author: Stephane Poss
"""

from __future__ import print_function
from __future__ import absolute_import
__RCSID__ = "$Id$"
# pylint: skip-file
# pylint: disable=C0103
from DIRAC.Core.Base import Script
Script.parseCommandLine()

from ILCDIRAC.Interfaces.API.NewInterface.SIDProductionJob import SIDProductionJob
from ILCDIRAC.Interfaces.API.NewInterface.Applications import OverlayInput
from ILCDIRAC.Interfaces.API.NewInterface.Applications import SLIC, LCSIM, SLICPandora, SLCIOSplit, StdHepSplit, Marlin
from ILCDIRAC.Interfaces.API.DiracILC import DiracILC

dirac = DiracILC()

# As it's a full chain, we start at generation
# so we need to define the process and the energy
# The rest will be set later. We could also set the process
# and the energy directly in the whizard def, but for clarity
# it's better to do it before, that way we know the very
# essential


analysis = 'tth_siddbd'
process = 'tth-6q-hbb'
additional_name = ''
energy = 1000.  # Needed for overlay
meta_energy = '1000'

# For meta def
meta = {}
# meta['ProdID']=838
meta['EvtType'] = process
#meta['Energy'] = meta_energy
meta['Datatype'] = 'GEN'
meta['Polarisation'] = 'm80p20'
meta['Machine'] = 'ilc'
meta['DetectorType'] = 'sid'

detector_model = "clic_sid_cdr"  # Once set, no need to chnage it, all prods should use the same (except bugs)
tracking_strategy = "defaultStrategies_clic_sid_cdr.xml"  # Once set, never change (EVER)
# DoSplit
activesplitstdhep = False
nbevtsperfilestdhep = 100

# Do Sim
sid_sim = True

# DoSplit
activesplit = False
nbevtsperfile = 200

# Do Reco
sid_rec = True
# Do Reco with Overlay
sid_rec_ov = False

n_events = 100  # How many events per job. (Is also obtained from the FC in principle)


# Split
stdhepsplit = StdHepSplit()
stdhepsplit.setVersion("V2")
stdhepsplit.setNumberOfEventsPerFile(nbevtsperfilestdhep)


# Simulation SID
slic = SLIC()
slic.setVersion('v2r9p8')  # This will change only once
slic.setSteeringFile('defaultClicCrossingAngle.mac')  # This will change only once
slic.setDetectorModel(detector_model)
slic.setNumberOfEvents(n_events)

# Split
split = SLCIOSplit()
split.setNumberOfEventsPerFile(nbevtsperfile)

# SID Reco w/o overlay
lcsim_prepandora = LCSIM()
lcsim_prepandora.setVersion('CLIC_CDR')  # This will change only once
lcsim_prepandora.setSteeringFile("clic_cdr_prePandora.lcsim")  # The steering files should NOT CHANGE
lcsim_prepandora.setTrackingStrategy(tracking_strategy)
# lcsim_prepandora.setDetectorModel(detector_model)
lcsim_prepandora.setOutputFile("prePandora.slcio")  # NEVER CHANGE THIS, this file is not stored in any case
lcsim_prepandora.setNumberOfEvents(n_events)
lcsim_prepandora.willRunSLICPandora()

slicpandora = SLICPandora()
slicpandora.setVersion('CLIC_CDR')  # This will change only once
slicpandora.setDetectorModel(detector_model)
slicpandora.setPandoraSettings("PandoraSettingsSlic.xml")
slicpandora.getInputFromApp(lcsim_prepandora)
slicpandora.setOutputFile('pandora.slcio')  # NEVER CHANGE THIS, this file is not stored in any case

lcfivertex = Marlin()
lcfivertex.setVersion()
lcfivertex.getInputFromApp(slicpandora)
lcfivertex.setSteeringFile()
lcfivertex.setOutputFile("lcfivertex.slcio")  # NEVER CHANGE THIS, this file is not stored in any case

# Final step. Outpufile is resolved automatically
lcsim_postpandora = LCSIM()
lcsim_postpandora.setVersion('CLIC_CDR')  # This will change only once
lcsim_postpandora.getInputFromApp(lcfivertex)
lcsim_postpandora.setSteeringFile("clic_cdr_postPandoraOverlay.lcsim")  # This will change only once
lcsim_postpandora.setTrackingStrategy(tracking_strategy)
# lcsim_postpandora.setDetectorModel(detector_model)

# SID Reco w/o overlay
overlay_sid_gg = OverlayInput()
overlay_sid_gg.setBkgEvtType("gghad")  # DO NOT TOUCH THIS
overlay_sid_gg.setEnergy(energy)
if energy == 1000.:
  overlay_sid_gg.setBXOverlay(1)
  overlay_sid_gg.setGGToHadInt(4.1)  # When running at 3TeV
else:
  print("Overlay SID: No overlay parameters defined for this energy")
overlay_sid_gg.setDetectorModel("sidloi3")  # DO NOT TOUCH THIS

overlay_sid_pairs = OverlayInput()
overlay_sid_pairs.setBkgEvtType("pairs")  # DO NOT TOUCH THIS
overlay_sid_pairs.setEnergy(energy)
if energy == 1000.:
  overlay_sid_pairs.setBXOverlay(1)
  overlay_sid_pairs.setGGToHadInt(1)  # When running at 3TeV
else:
  print("Overlay SID: No overlay parameters defined for this energy")
overlay_sid_pairs.setDetectorModel("sidloi3")  # DO NOT TOUCH THIS


lcsim_prepandora_ov = LCSIM()
lcsim_prepandora_ov.setVersion('CLIC_CDR')  # This will change only once
if energy == 1000.0:
  lcsim_prepandora_ov.setSteeringFile("clic_cdr_prePandoraOverlay_3000.0.lcsim")  # This will change only once
else:
  print("LCSIM: No steering files defined for this energy")

lcsim_prepandora_ov.setTrackingStrategy(tracking_strategy)
# lcsim_prepandora_ov.setDetectorModel(detector_model)
lcsim_prepandora_ov.setOutputFile("prePandora.slcio")  # NEVER CHANGE THIS, this file is not stored in any case
lcsim_prepandora_ov.willRunSLICPandora()

slicpandora_ov = SLICPandora()
slicpandora_ov.getInputFromApp(lcsim_prepandora_ov)
slicpandora_ov.setVersion('CLIC_CDR')  # This will change only once
slicpandora_ov.setDetectorModel(detector_model)
slicpandora_ov.setPandoraSettings("PandoraSettingsSlic.xml")  # This will change only once
slicpandora_ov.setOutputFile('pandora.slcio')  # NEVER CHANGE THIS, this file is not stored in any case

lcfivertex_ov = Marlin()
lcfivertex_ov.setVersion()
lcfivertex_ov.getInputFromApp(slicpandora_ov)
lcfivertex_ov.setSteeringFile()
lcfivertex_ov.setOutputFile("lcfivertex.slcio")  # NEVER CHANGE THIS, this file is not stored in any case

# Final step. Outpufile is resolved automatically
lcsim_postpandora_ov = LCSIM()
lcsim_postpandora_ov.getInputFromApp(lcfivertex_ov)
lcsim_postpandora_ov.setVersion('CLIC_CDR')  # This will change only once
lcsim_postpandora_ov.setSteeringFile("clic_cdr_postPandoraOverlay.lcsim")  # This will change only once
lcsim_postpandora_ov.setTrackingStrategy(tracking_strategy)
# lcsim_postpandora_ov.setDetectorModel(detector_model)

############################################
# What is below WILL NEVER NEED TO BE TOUCHED
# (I'm not kidding, if you touch and break, not my problem)
#
if activesplitstdhep and meta:
  pstdhepsplit = SIDProductionJob()
  pstdhepsplit.setLogLevel("verbose")
  pstdhepsplit.setProdType('Split')
  res = pstdhepsplit.setInputDataQuery(meta)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pstdhepsplit.setOutputSE("CERN-SRM")
  wname = process + "_" + str(energy) + "_split"
  wname += additional_name
  pstdhepsplit.setWorkflowName(wname)
  pstdhepsplit.setProdGroup(analysis + "_" + str(energy))

  # Add the application
  res = pstdhepsplit.append(stdhepsplit)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  pstdhepsplit.addFinalization(True, True, True, True)
  descrp = "Splitting stdhep files"
  if additional_name:
    descrp += ", %s" % additional_name
  pstdhepsplit.setDescription(descrp)

  res = pstdhepsplit.createProduction()
  if not res['OK']:
    print(res['Message'])
  res = pstdhepsplit.finalizeProd()
  if not res['OK']:
    print(res['Message'])
    exit(1)
  # As before: get the metadata for this production to input into the next
  meta = pstdhepsplit.getMetadata()


if sid_sim and meta:
  ####################
  # Define the second production (simulation). Notice the setInputDataQuery call
  psl = SIDProductionJob()
  psl.setLogLevel("verbose")
  psl.setProdType('MCSimulation')
  res = psl.setInputDataQuery(meta)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  psl.setOutputSE("CERN-SRM")
  wname = process + "_" + str(energy) + "_sid_sim"
  wname += additional_name
  psl.setWorkflowName(wname)
  psl.setProdGroup(analysis + "_" + str(energy))
  # Add the application
  res = psl.append(slic)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  psl.addFinalization(True, True, True, True)
  descrp = "CLIC_SID_CDR model"
  if additional_name:
    descrp += ", %s" % additional_name
  psl.setDescription(descrp)

  res = psl.createProduction()
  if not res['OK']:
    print(res['Message'])
  res = psl.finalizeProd()
  if not res['OK']:
    print(res['Message'])
    exit(1)
  # As before: get the metadata for this production to input into the next
  meta = psl.getMetadata()

if activesplit and meta:
  #######################
  # Split the input files.
  psplit = SIDProductionJob()
  psplit.setCPUTime(30000)
  psplit.setLogLevel("verbose")
  psplit.setProdType('Split')
  psplit.setDestination("LCG.CERN.ch")
  res = psplit.setInputDataQuery(meta)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  psplit.setOutputSE("CERN-SRM")
  wname = process + "_" + str(energy) + "_split"
  wname += additional_name
  psplit.setWorkflowName(wname)
  psplit.setProdGroup(analysis + "_" + str(energy))

  # Add the application
  res = psplit.append(split)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  psplit.addFinalization(True, True, True, True)
  descrp = "Splitting slcio files"
  if additional_name:
    descrp += ", %s" % additional_name
  psplit.setDescription(descrp)

  res = psplit.createProduction()
  if not res['OK']:
    print(res['Message'])
  res = psplit.finalizeProd()
  if not res['OK']:
    print(res['Message'])
    exit(1)
  # As before: get the metadata for this production to input into the next
  meta = psplit.getMetadata()


if sid_rec and meta:
  #######################
  # Define the reconstruction prod
  psidrec = SIDProductionJob()
  psidrec.setLogLevel("verbose")
  psidrec.setProdType('MCReconstruction')
  psidrec.setBannedSites(['LCG.Bristol.uk', 'LCG.RAL-LCG2.uk'])
  res = psidrec.setInputDataQuery(meta)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  psidrec.setOutputSE("CERN-SRM")
  wname = process + "_" + str(energy) + "_sid_rec"
  wname += additional_name
  psidrec.setWorkflowName(wname)
  psidrec.setProdGroup(analysis + "_" + str(energy))
  res = psidrec.append(lcsim_prepandora)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  res = psidrec.append(slicpandora)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  res = psidrec.append(lcsim_postpandora)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  psidrec.addFinalization(True, True, True, True)
  descrp = "CLIC_SID_CDR, No overlay"
  if additional_name:
    descrp += ", %s" % additional_name
  psidrec.setDescription(descrp)

  res = psidrec.createProduction()
  if not res['OK']:
    print(res['Message'])
  res = psidrec.finalizeProd()
  if not res['OK']:
    print(res['Message'])
    exit(1)

if sid_rec_ov and meta:
  #######################
  # Define the reconstruction prod
  psidreco = SIDProductionJob()
  psidreco.setLogLevel("verbose")
  psidreco.setProdType('MCReconstruction_Overlay')
  psidreco.setBannedSites(['LCG.Bristol.uk', 'LCG.RAL-LCG2.uk'])
  res = psidreco.setInputDataQuery(meta)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  psidreco.setOutputSE("CERN-SRM")
  wname = process + "_" + str(energy) + "_sid_rec_overlay"
  wname += additional_name
  psidreco.setWorkflowName(wname)
  psidreco.setProdGroup(analysis + "_" + str(energy))
  res = psidreco.append(overlay_sid_gg)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  res = psidreco.append(overlay_sid_pairs)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  res = psidreco.append(lcsim_prepandora_ov)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  res = psidreco.append(slicpandora_ov)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  res = psidreco.append(lcsim_postpandora_ov)
  if not res['OK']:
    print(res['Message'])
    exit(1)
  psidreco.addFinalization(True, True, True, True)
  descrp = "CLIC_SID_CDR, overlay"
  if additional_name:
    descrp += ", %s" % additional_name
  psidreco.setDescription(descrp)

  res = psidreco.createProduction()
  if not res['OK']:
    print(res['Message'])
  res = psidreco.finalizeProd()
  if not res['OK']:
    print(res['Message'])
    exit(1)

# In principle nothing else is needed.



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Class wrapping around all Whizard 1.95 options.

For SUSY special treatment of the pythia parameters have to be thought of: if the model is SUSY, then need to add
IMSS(1)=11; IMSS(21)=71; IMSS(22)=71;

This is currently done on the job definition side

:since:  Nov 3, 2011
:author: Stephane Poss
"""

from __future__ import print_function
from __future__ import absolute_import
from xml.etree.ElementTree import ElementTree, fromstring
from ILCDIRAC.Core.Utilities.GeneratorModels import GeneratorModels

from DIRAC import S_OK, S_ERROR, gLogger
import six
from six.moves import range

LOG = gLogger.getSubLogger(__name__)


def getDict():
  """Get list of available fields in the whizard options."""
  pdict = {}
  pdict['process_input'] = {}
  pdict['integration_input'] = {}
  pdict['simulation_input'] = {}
  pdict['diagnostics_input'] = {}
  pdict['beam_input_1'] = {}
  pdict['beam_input_2'] = {}

  return S_OK(pdict)


class WhizardOptions(object):
  """Class that provides an interface to the xml representation of the whizard options."""

  def __init__(self, model="sm"):
    """
    :param str model: physics model to be used by whizard, default "sm"
    """

    self.genmodel = GeneratorModels()
    self.paramdict = {}
    modelparams = self.modelParams(model)
    self.whizardxml = fromstring("""<whizard>
<process_input>
<process_id type="string" value="">
<!-- Process tag(s) as defined in whizard.prc. It should contain the list of processes to activate, separated by commas or blanks, enclosed in quotes. -->
</process_id>
<cm_frame type="T/F" value="T">
<!-- if true, the c.m. frame is the lab frame, the beams are in +/- z directions, and the total c.m. energy is given by sqrts. If false, the beam energies and directions must be specified below in the blocks beam_input. -->
</cm_frame>
<sqrts type="float" value="3000">
<!-- If this number is greater than the sum of the incoming particle masses, it specifies the c.m. energy of the initial state in GeV. Applies only if cm_frame is true, and is ignored for decay processes. -->
</sqrts>
<luminosity type="float" value="0">
<!-- Integrated luminosity in fb-1 -->
</luminosity>
<polarized_beams type="T/F" value="T">
<!-- If true, the helicity content of the beams must be specified below in the blocks beam_input. -->
</polarized_beams>
<structured_beams type="T/F" value="T">
<!-- If true, the nature of the incoming beams must be specified below in the blocks beam_input. -->
</structured_beams>
<beam_recoil type="T/F" value="F">
<!-- If true, and if structure functions (e.g., ISR) are selected, the recoil of the partons against the beam remnant (e.g., the emitted photons) is taken into account. The pT distribution is computed within the approximation valid for the emission, hence it will be accurate at low pT. -->
</beam_recoil>
<recoil_conserve_momentum type="T/F" value="F">
<!-- Applies only if beam_recoil is set: if true, keep momentum balance between parton and recoil momenta at the expense of energy balance. If false, keep energy balance at the expense of momentum balance. -->
</recoil_conserve_momentum>
<filename type="string" value="whizard">
<!-- Base filename (w/o extension) to be used for all input/output files instead of the string "whizard". -->
</filename>
<directory type="string" value="">
<!-- Working directory for all further reading/writing of files. -->
</directory>
<input_file type="string" value="">
<!-- If nonempty, read the specified input file after the current one. The extension .in will be appended to the filename. -->
</input_file>
<input_slha_format type="T/F" value="F">
<!-- If true, assume that the next input file is in SUSY Les Houches Accord format (see Sec. 4.4.7). If false, determine the format from the first line. -->
</input_slha_format>
</process_input>
<integration_input>
<calls type="floatarray" value="1 50000 10 50000 1 1500000">
<!-- Array describing the number of iterations and number of calls per integration pass. Default values depend on the selected process. See below in subsection 4.6 for details. -->
</calls>
<seed type="integer" value="">
<!-- Random number generator seed (integer). When omitted, the time counter will be used, resulting in a different value each run. -->
</seed>
<reset_seed_each_process type="T/F" value="F">
<!-- Reset the random number generator seed to seed not just once, but each time a process is integrated. This is useful for comparing matrix elements which should be identical, but, e.g., have been generated by different programs. -->
</reset_seed_each_process>
<accuracy_goal type="float" value="0">
<!-- Goal for the accuracy estimate (6th column in the output). When this goal is reached and the efficiency goal is either also reached or unset, further grid adaptation iterations will be skipped (Sec. 4.9). -->
</accuracy_goal>
<efficiency_goal type="float" value="100">
<!-- Goal for the reweighting efficiency estimate (7th column in the output). When this goal is reached and the accuracy goal is either also reached or unset, further grid adaptation iterations will be skipped (Sec. 4.9). -->
</efficiency_goal>
<time_limit_adaptation type="integer" value="0">
<!-- If nonzero, grid adaptation for the current process will be stopped after the specified number of minutes, and the final integration pass started (Sec. 4.9). -->
</time_limit_adaptation>
<stratified type="T/F" value="T">
<!-- Use stratified (T) / importance (F) sampling. -->
</stratified>
<use_efficiency type="T/F" value="F">
<!-- Use efficiency (T) / accuracy (F) as the criterion for adapting the channel weights. -->
</use_efficiency>
<weights_power type="float" value="0.25">
<!-- Power used for adapting the channel weights. Lower value means slower adaptation (to suppress fluctuations). -->
</weights_power>
<min_bins type="integer" value="3">
<!-- Minimal number of bins per integration dimension. -->
</min_bins>
<max_bins type="integer" value="20">
<!-- Maximal number of bins per integration dimension. This number will be used as long as there are enough sampling points, otherwise the number of bins will be decreased. -->
</max_bins>
<min_calls_per_bin type="integer" value="10">
<!-- Minimal number of points per bin, integration dimension, and integration channel. If this limit cannot be satisfied, the total number of points will be increased. -->
</min_calls_per_bin>
<min_calls_per_channel type="integer" value="0">
<!-- All integration channel will get at least (approximately) this number of points. Prevents channels from being dropped during adaptation. -->
</min_calls_per_channel>
<write_grids type="T/F" value="T">
<!-- Write grids to files whizard.grb (best grid) and whizard.grc (current grid), to be reused later. -->
</write_grids>
<write_grids_raw type="T/F" value="F">
<!-- Use binary format for writing grids. Saves memory at the expense of portability. -->
</write_grids_raw>
<write_grids_file type="string" value="">
<!-- If nonempty, use the specified filename for writing grids instead of the default. The file extensions are appended to string. -->
</write_grids_file>
<write_all_grids type="T/F" value="F">
<!-- Write, in addition, after each iteration the current grid to file whizard.grXX, where XX is the iteration number. -->
</write_all_grids>
<write_all_grids_file type="string" value="">
<!-- If nonempty, use the specified filename for writing the extra grids instead of the default. The file extensions are appended to string. -->
</write_all_grids_file>
<read_grids type="T/F" value="F">
<!-- Read existing grids whizard.grb and whizard.grc if they have been written by a previous run. This avoids the time-consuming adaptation step. Makes sense only if no physical parameters have been changed. -->
</read_grids>
<read_grids_raw type="T/F" value="F">
<!-- If true, search first for binary grid files, then for ASCII grids. If false, do search first for ASCII.. -->
</read_grids_raw>
<read_grids_force type="T/F" value="T">
<!-- Set this to T if you want to read the grids from file even if some parameters have changed. Use with care! This may result in a program crash if the grid structures are incompatible. -->
</read_grids_force>
<read_grids_file type="string" value="">
<!-- If nonempty, use the specified filename for reading grids instead of the default. The file extensions are appended to string. -->
</read_grids_file>
<generate_phase_space type="T/F" value="T">
<!-- Generate a phase space configuration appropriate for the current process and write it to whizard.phx. -->
</generate_phase_space>
<read_model_file type="string" value="">
<!-- If nonempty, read vertex definitions for phase space setup from string.mdl instead of the default whizard.mdl. -->
</read_model_file>
<write_phase_space_file type="string" value="">
<!-- Write phase space configuration to string.phx instead of the default. -->
</write_phase_space_file>
<read_phase_space type="T/F" value="T">
<!-- Read phase space configuration from whizard.phs or a previously generated file whizard.phx if possible. -->
</read_phase_space>
<read_phase_space_file type="string" value="">
<!-- Read phase space configuration from string.phs or string.phx instead of the default. -->
</read_phase_space_file>
<phase_space_only type="T/F" value="F">
<!-- Stop the program after phase space generation. -->
</phase_space_only>
<use_equivalences type="T/F" value="T">
<!-- If true, use permutation symmetry when updating grids to improve the quality of the results. -->
</use_equivalences>
<azimuthal_dependence type="T/F" value="F">
<!-- If false, it is assumed that the scattering does not depend on the overall azimuthal angle. This will be automatically T if general beam polarization is switched on, therefore the user need only access this parameter in the case of azimuthal-dependent cuts. -->
</azimuthal_dependence>
<write_phase_space_channels_file type="string" value="">
<!-- Show phase space channels in string.ps instead of the default file whizard-channels.ps. Note that you need to call CHANNELS= string make -e channels in order to generate string.ps -->
</write_phase_space_channels_file>
<off_shell_lines type="integer" value="1">
<!-- Maximum number of off-shell-lines allowed for Feynman graphs which are initially taken into account for the phase space configuration. Log-enhanced (massless) propagators are not counted as off-shell. -->
</off_shell_lines>
<extra_off_shell_lines type="integer" value="1">
<!-- Use configurations with more off-shell lines, if they happen to be maximally resonant. -->
</extra_off_shell_lines>
<splitting_depth type="integer" value="1">
<!-- Up to this number of branchings, a (massless) propagator will be considered as log-enhanced and mapped like a photon propagator. -->
</splitting_depth>
<exchange_lines type="integer" value="3">
<!-- Up to this number of t-channel propagators, a multiperipheral graph will be taken into account. -->
</exchange_lines>
<show_deleted_channels type="T/F" value="F">
<!-- With extra_off_shell_lines, extra channels will be generated which are deleted if they do not contain enough resonances. With this flag, they are just commented out, so they could be manually activated. -->
</show_deleted_channels>
<single_off_shell_decays type="T/F" value="T">
<!-- Whether single-off-shell decays are relevant for the phase space configuration. -->
</single_off_shell_decays>
<double_off_shell_decays type="T/F" value="F">
<!-- Whether double-off-shell decays are relevant for the phase space configuration. -->
</double_off_shell_decays>
<single_off_shell_branchings type="T/F" value="T">
<!-- Whether single-off-shell branchings are relevant for the phase space configuration. -->
</single_off_shell_branchings>
<double_off_shell_branchings type="T/F" value="T">
<!-- Whether double-off-shell branchings are relevant for the phase space configuration. -->
</double_off_shell_branchings>
<massive_fsr type="T/F" value="T">
<!-- Whether the radiation of a massive particle in the final state is relevant for the phase space configuration. -->
</massive_fsr>
<threshold_mass type="float" value="-10">
<!-- A particle with a mass up to this value will be considered as massless for the purpose of phase-space setup. (But the true mass is taken into account when the particle appears as a resonant intermediate state.) -->
</threshold_mass>
<threshold_mass_t type="float" value="-10">
<!-- A particle with a mass up to this value will be considered as massless for the purpose of phase-space setup, when it appears as a t-channel propagator. -->
</threshold_mass_t>
<!-- <initial_decays_fatal type="T/F" value="T"> -->
<!-- As the phase space maps cannot describe on-shell decays of beam particles properly, WHIZARD normally gives a fatal error when such configurations are encountered. This option changes this to a warning at the price of a potentially screwed phase space setup. -->
<!-- </initial_decays_fatal> -->
<default_jet_cut type="float" value="10">
<!-- The default invariant mass cut in GeV applied to pairs of massless colored particles. -->
</default_jet_cut>
<default_mass_cut type="float" value="4">
<!-- The default invariant mass cut in GeV applied to pair production of massless colorless charged particles and to photon emission. -->
</default_mass_cut>
<default_energy_cut type="float" value="10">
<!-- The default energy cut in GeV applied to photon and gluon emission. -->
</default_energy_cut>
<default_q_cut type="float" value="4">
<!-- The default Q cut in GeV applied to photon and gluon exchange. -->
</default_q_cut>
<write_default_cuts_file type="string" value="">
<!-- If nonempty, write the list of default cuts to this file (augmented by the file extension) instead of the default. Note that the settings in this file are overwritten by a user-defined cut configuration, if present. -->
</write_default_cuts_file>
<read_cuts_file type="string" value="">
<!-- Look for user-defined cut configurations in string.cut1 instead of whizard.cut1. -->
</read_cuts_file>
<user_cut_mode type="integer" value="0">
<!-- Set this nonzero to activate a user-defined cut function (Sec. 5.4). -->
</user_cut_mode>
<user_weight_mode type="integer" value="0">
<!-- Set this nonzero to activate a user-defined weight function (Sec. 5.5). -->
</user_weight_mode>
</integration_input>
<simulation_input>
<n_events type="integer" value="0">
<!-- Number of (unweighted) events to generate at least, irrespective of the luminosity setting. -->
</n_events>
<n_calls type="integer" value="0">
<!-- Number of matrix-element calls (weighted events) to execute at least, irrespective of the luminosity setting. -->
</n_calls>
<n_events_warmup type="integer" value="0">
<!-- Number of extra warmup events (see below, Sec. 4.7). -->
</n_events_warmup>
<unweighted type="T/F" value="T">
<!-- Reweight events to generate an unweighted event sample. -->
</unweighted>
<normalize_weight type="T/F" value="T">
<!-- If true, normalize the event weight to unity. If false, normalize to the total cross section. -->
</normalize_weight>
<write_weights type="T/F" value="F">
<!-- If unweighted=F, write weight distribution data to whizard.wgt. -->
</write_weights>
<write_weights_file type="string" value="">
<!-- Write weight distribution to string.wgt instead. -->
</write_weights_file>
<safety_factor type="float" value="1">
<!-- Multiply the estimate for the highest weight by this factor before starting event generation. -->
</safety_factor>
<write_events type="T/F" value="T">
<!-- Write generated events to file whizard.evt to be used by an external analysis package. -->
</write_events>
<write_events_format type="integer" value="20">
<!-- The format to be used for writing events, where the file extension depends on the format (.evt for format = 1, see Sec. 4.7). -->
</write_events_format>
<write_events_file type="string" value="">
<!-- If nonempty, use string as filename for writing events, where the file extension will be appended. -->
</write_events_file>
<events_per_file type="integer" value="5000000">
<!-- If positive, begin a new event file once the number of entries exceeds this number. The event file counter is appended to each event file name, separated with a dot (before the file extension). This feature applies only to non-binary event formats. -->
</events_per_file>
<bytes_per_file type="integer" value="0">
<!-- If positive, begin a new event file once the number of bytes in the file exceeds this number. The event file counter is appended to each event file name, separated with a dot (before the file extension). This feature applies only to non-binary event formats. See Sec. 4.7.1. -->
</bytes_per_file>
<min_file_count type="integer" value="1">
<!-- If event files are split, use this index for the first event file. Increase the counter by one for each successive event file. -->
</min_file_count>
<max_file_count type="integer" value="999">
<!-- Limit for the event file counter; if this limit is exceeded, event generation is terminated. (For weighted events only, this is an error condition since the event sample must be complete for being usable.) -->
</max_file_count>
<write_events_raw type="T/F" value="F">
<!-- Write events to whizard.evx in condensed binary format, so they can be internally reused in another run. -->
</write_events_raw>
<write_events_raw_file type="string" value="">
<!-- Write raw events to string.evx instead. -->
</write_events_raw_file>
<read_events type="T/F" value="F">
<!-- Read events from file whizard.evx (raw format) instead of generating them. This is equivalent to read_events_raw. -->
</read_events>
<read_events_force type="T/F" value="T">
<!-- This was intended to force WHIZARD to read in events from file even if some parameters have changed. However, the MD5 checksum implemented to check for parameter changes has some deficiencies. Hence, we always enforce to read in events, because this feature could otherwise not be used at all. Use with great care! (Note that this problem has been solved in WHIZARD 2.) -->
</read_events_force>
<read_events_raw_file type="string" value="">
<!-- Read raw events from string.evx instead. -->
</read_events_raw_file>
<keep_beam_remnants type="T/F" value="T">
<!-- Keep the beam remnants in the event record when applying structure functions. See Sec. 4.4.8. -->
</keep_beam_remnants>
<keep_initials type="T/F" value="T">
<!-- Keep the beam particles and the partons which initiate the hard scattering in the event record. See Sec. 4.4.8. -->
</keep_initials>
<guess_color_flow type="T/F" value="F">
<!-- Infer the color flow for hadronization from the particle ordering, if it is nontrivial and not available directly. -->
</guess_color_flow>
<recalculate type="T/F" value="F">
<!-- Recalculate the matrix element value for each event of a previously generated sample. Setting this flag automatically turns on reading grids and events from file. -->
</recalculate>
<fragment type="T/F" value="T">
<!-- Fragment the events depending on the value of fragmentation_method (see Sec. 4.8). -->
</fragment>
<fragmentation_method type="integer" value="3">
<!-- Method used for fragmentation if fragment is true: 0=no fragmentation; 1=JETSET; 2=PYTHIA; 3=user. -->
</fragmentation_method>
<user_fragmentation_mode type="integer" value="0">
<!-- When user-defined fragmentation routines are called, this parameter may select different modes. -->
</user_fragmentation_mode>
<pythia_parameters type="string" value="PMAS(25,1)=120.; PMAS(25,2)=0.3605E-02; MSTU(22)=20 ;MSTJ(28)=2 ;PARJ(21)=0.40000;PARJ(41)=0.11000; PARJ(42)=0.52000; PARJ(81)=0.25000; PARJ(82)=1.90000; MSTJ(11)=3; PARJ(54)=-0.03100; PARJ(55)=-0.00200;PARJ(1)=0.08500; PARJ(3)=0.45000; PARJ(4)=0.02500; PARJ(2)=0.31000; PARJ(11)=0.60000; PARJ(12)=0.40000; PARJ(13)=0.72000;PARJ(14)=0.43000; PARJ(15)=0.08000; PARJ(16)=0.08000; PARJ(17)=0.17000; MSTP(3)=1;">
<!-- String to be given to PYTHIA's pygive call before starting event generation. This allows to modify PYTHIA/JETSET properties, set particle masses, etc. The string is also available within user-defined fragmentation routines and can there be abused for different purposes. -->
</pythia_parameters>
<pythia_processes type="string" value="">
<!-- PYTHIA background processes to be simulated in addition to the WHIZARD processes: A list of integers separated by blanks, enclosed in quotation marks. Refer to the PYTHIA manual for the list of processes. -->
</pythia_processes>
<shower type="T/F" value="F">
<!-- Switches the internal shower on or off. As a default it is off. Note that the shower only works with LHEF event format (foramt type 6). This shower is highly experimental. Mainly intended for testing purposes. -->
</shower>
<shower_nf type="integer" value="5">
<!-- Number of light flavors in the shower. -->
</shower_nf>
<shower_running_alpha_s type="T/F" value="F">
<!-- Whether to use a running strong coupling alpha_s or not in the shower. As a default it is fixed. -->
</shower_running_alpha_s>
<shower_alpha_s type="float" value="0.2">
<!-- The value of the strong coupling constant alpha_s as used by the internal shower. The default is 0.2. -->
</shower_alpha_s>
<shower_lambda type="float" value="0.29">
<!-- The value of the QCD scale Lambda_QCD as used by the internal shower. The default is 0.29 GeV. -->
</shower_lambda>
<shower_t_min type="float" value="1.0">
<!-- The infrared cutoff for the evolution parameter tmin as used by the internal shower. The default is 1 GeV. -->
</shower_t_min>
<shower_md type="float" value="0.330">
<!-- The constituent d quark mass md as used by the internal shower. The default is 0.330 GeV. -->
</shower_md>
<shower_mu type="float" value="0.330">
<!-- The constituent u quark mass mu as used by the internal shower. The default is 0.330 GeV. -->
</shower_mu>
<shower_ms type="float" value="0.500">
<!-- The constituent s quark mass ms as used by the internal shower. The default is 0.5 GeV. -->
</shower_ms>
<shower_mc type="float" value="1.5">
<!-- The constituent c quark mass mc as used by the internal shower. The default is 1.5 GeV. -->
</shower_mc>
<shower_mb type="float" value="4.8">
<!-- The constituent b quark mass mb as used by the internal shower. The default is 4.8 GeV.diagnostics_input -->
</shower_mb>
</simulation_input>
<diagnostics_input>
<chattiness type="integer" value="4">
<!-- How much information to show on screen: (0) only fatal errors, (1) and non-fatal errors, (2) and warnings, (3) and messages, (4) and results, (5) and debugging messages (if any). -->
</chattiness>
<catch_signals type="T/F" value="T">
<!-- If the compiler supports it, try to catch external signals such as SIGINT and SIGXCPU and exit gracefully, closing files first. -->
</catch_signals>
<time_limit type="integer" value="0"> -->
<!-- If nonzero, exit gracefully after the given number of minutes has passed. This is useful to prevent an external kill within a batch environment. -->
</time_limit>
<warn_empty_channel type="T/F" value="F">
<!-- Issue a warning whenever the integral within a phase space channel is zero. -->
</warn_empty_channel>
<screen_events type="T/F" value="F">
<!-- Whether to show generated events on screen. -->
</screen_events>
<screen_histograms type="T/F" value="F">
<!-- Whether to show histograms on screen. -->
</screen_histograms>
<screen_diagnostics type="T/F" value="F">
<!-- Whether to repeat the input parameters on screen. -->
</screen_diagnostics>
<show_pythia_banner type="T/F" value="T">
<!-- Whether to display the PYTHIA banner page if fragmentation is enabled. -->
</show_pythia_banner>
<show_pythia_initialization type="T/F" value="T">
<!-- Whether to display the PYTHIA initialization messages if fragmentation is enabled. -->
</show_pythia_initialization>
<show_pythia_statistics type="T/F" value="T">
<!-- Whether to display the PYTHIA statistics summary after event generation is completed. -->
</show_pythia_statistics>
<write_logfile type="T/F" value="T">
<!-- Whether to write the (process-specific) output file(s) whizard.XX.out. -->
</write_logfile>
<write_logfile_file type="string" value="">
<!-- Use this as the filename for the logfile. -->
</write_logfile_file>
<show_input type="T/F" value="T">
<!-- Whether to repeat the input parameters in the logfile. -->
</show_input>
<show_results type="T/F" value="T">
<!-- Whether to show the integration results in namelist format in the logfile. -->
</show_results>
<show_phase_space type="T/F" value="F">
<!-- Whether to show the phase space configuration in the logfile. -->
</show_phase_space>
<show_cuts type="T/F" value="T">
<!-- Whether to show the cut configuration in the logfile. -->
</show_cuts>
<show_histories type="T/F" value="F">
<!-- Whether to show the individual VAMP channel histories in the logfile. -->
</show_histories>
<show_history type="T/F" value="T">
<!-- Whether to show the overall VAMP history in the logfile. -->
</show_history>
<show_weights type="T/F" value="T">
<!-- Whether to show the weight adaptation in the logfile. -->
</show_weights>
<show_event type="T/F" value="F">
<!-- Whether to show the last event in the logfile. -->
</show_event>
<show_histograms type="T/F" value="F">
<!-- Whether to show histograms in the logfile. -->
</show_histograms>
<show_overflow type="T/F" value="F">
<!-- Whether to show events beyond the first or last bin in histogram listings. -->
</show_overflow>
<show_excess type="T/F" value="T">
<!-- Whether to show a summary of events with weight exceeding one. -->
</show_excess>
<read_analysis_file type="string" value="">
<!-- Use this (string.cut5) as the filename for the analysis setup instead of whizard.cut5 -->
</read_analysis_file>
<plot_width type="float" value="130">
<!-- The width in mm of the plots if online analysis is enabled. -->
</plot_width>
<plot_height type="float" value="90">
<!-- The height in mm of the plots if online analysis is enabled. -->
</plot_height>
<plot_excess type="T/F" value="T">
<!-- In the plots, display excess events in red. -->
</plot_excess>
<plot_history type="T/F" value="T">
<!-- If this is enabled, write a graphics driver file for displaying the integration history, i.e., the integral with error bars for each iteration. Use make history to generate the graphics file whizard-history.ps. -->
</plot_history>
<plot_grids_channels type="string" value="">
<!-- The string is a list of phase-space channels (integers) for which the bin distribution will be histogrammed. Use make grids to generate the graphics file whizard-grids.ps. -->
</plot_grids_channels>
<plot_grids_logscale type="float" value="10">
<!-- Use logarithmic scale for the grid plots if the bin width varies over more than this ratio. -->
</plot_grids_logscale>
<slha_rewrite_input type="T/F" value="T">
<!-- If SUSY Les Houches Accord data have been used, whether to repeat this input in the process-specific logfiles (including comments) or to rewrite it there using the data which have actually been used. -->
</slha_rewrite_input>
<slha_ignore_errors type="T/F" value="F">
<!-- If this is false, an error signaled in the SLHA input file (in the SPINFO or DCINFO block) will cause WHIZARD to stop before calculating anything. If true, such errors will be displayed, but the run continues. -->
</slha_ignore_errors>
</diagnostics_input>
%s
<beam_input_1>
<energy type="float" value="0">
<!-- If greater than the beam particle mass, this specifies the beam energy in the lab frame. Otherwise, the beam energy is set equal to the particle mass (fixed target). -->
</energy>
<angle type="float" value="0">
<!-- If direction is not set, this specifies a rotation of the beam axis in the lab frame around the positive y axis. (By default, the beam directions are along the positive/negative z axis, so a rotation by the angle pi/2 turns them into the positive/negative x axis.) If direction is set, this parameter is ignored. -->
</angle>
<direction type="floatarray" value="0 0 0">
<!-- If any component is nonzero, this vector explicitly specifies the direction of the given beam in the lab frame. -->
</direction>
<vector_polarization type="T/F" value="F">
<!-- If false (default), use the standard helicity basis (left-/right-handed). Set this flag if you need another basis, in particular transversal polarization. -->
</vector_polarization>
<polarization type="floatarray" value="0.0 0.0">
<!-- Fraction of left/right polarization (fermions, photons, gluons), resp. left/longitudinal/right polarization (massive vector bosons). If the vector polarization model is selected, the three numbers denote the polarization vector. -->
</polarization>
<particle_code type="integer" value="0">
<!-- PDG code of the incoming beam particle. -->
</particle_code>
<particle_name type="string" value="e1">
<!-- Name of the incoming beam particle. -->
</particle_name>
<USER_spectrum_on type="T/F" value="T">
<!-- Apply or not the user spectrum -->
</USER_spectrum_on>
<USER_spectrum_mode type="integer" value="11">
<!-- User spectrum -->
</USER_spectrum_mode>
<ISR_on type="T/F" value="T">
<!-- Whether to apply ISR (electron or positron beam). -->
</ISR_on>
<ISR_alpha type="float" value="0.0072993">
<!-- The value of alpha_QED to be used for the spectrum. -->
</ISR_alpha>
<ISR_m_in type="float" value="0.000511">
<!-- The mass of the incoming particle. -->
</ISR_m_in>
<ISR_Q_max type="float" value="sqrts">
<!-- The hard scale which cuts off photon radiation. -->
</ISR_Q_max>
<ISR_LLA_order type="0/1/2/3" value="3">
<!-- The order of the leading-logarithmic approximation. -->
</ISR_LLA_order>
<ISR_map type="T/F" value="T">
<!-- Whether to use a mapping of the singularity at x=1 when evaluating the structure function (recommended; note that switching this off might even lead to an uncaught arithmetic exception). -->
</ISR_map>
<EPA_on type="T/F" value="F">
<!-- Whether to use the EPA spectrum (photon beam). -->
</EPA_on>
<EPA_map type="T/F" value="T">
<!-- Whether to apply a mapping to improve convergence. -->
</EPA_map>
<EPA_alpha type="float" value="0.0072993">
<!-- The value of alpha_QED to be used for the spectrum. -->
</EPA_alpha>
<EPA_m_in type="float" value="0.000511">
<!-- The mass of the incoming beam particle. -->
</EPA_m_in>
<EPA_mX type="float" value="4">
<!-- The lower cutoff for the produced invariant mass. -->
</EPA_mX>
<EPA_Q_max type="float" value="4">
<!-- The upper cutoff on the virtuality of the photon (Qmax&gt;0). -->
</EPA_Q_max>
<EPA_x0 type="float" value="0">
<!-- The lower cutoff on the energy fraction of the incoming photon -->
</EPA_x0>
<EPA_x1 type="float" value="0">
<!-- The upper cutoff on the energy fraction of the incoming photon -->
</EPA_x1>
</beam_input_1>
<beam_input_2>
<energy type="float" value="0">
<!-- If greater than the beam particle mass, this specifies the beam energy in the lab frame. Otherwise, the beam energy is set equal to the particle mass (fixed target). -->
</energy>
<angle type="float" value="0">
<!-- If direction is not set, this specifies a rotation of the beam axis in the lab frame around the positive y axis. (By default, the beam directions are along the positive/negative z axis, so a rotation by the angle pi/2 turns them into the positive/negative x axis.) If direction is set, this parameter is ignored. -->
</angle>
<direction type="floatarray" value="0 0 0">
<!-- If any component is nonzero, this vector explicitly specifies the direction of the given beam in the lab frame. -->
</direction>
<vector_polarization type="T/F" value="F">
<!-- If false (default), use the standard helicity basis (left-/right-handed). Set this flag if you need another basis, in particular transversal polarization. -->
</vector_polarization>
<polarization type="floatarray" value="0.0 0.0">
<!-- Fraction of left/right polarization (fermions, photons, gluons), resp. left/longitudinal/right polarization (massive vector bosons). If the vector polarization model is selected, the three numbers denote the polarization vector. -->
</polarization>
<particle_code type="integer" value="0">
<!-- PDG code of the incoming beam particle. -->
</particle_code>
<particle_name type="string" value="E1">
<!-- Name of the incoming beam particle. -->
</particle_name>
<USER_spectrum_on type="T/F" value="T">
<!-- Apply or not the user spectrum -->
</USER_spectrum_on>
<USER_spectrum_mode type="integer" value="-11">
<!-- User spectrum -->
</USER_spectrum_mode>
<ISR_on type="T/F" value="T">
<!-- Whether to apply ISR (electron or positron beam). -->
</ISR_on>
<ISR_alpha type="float" value="0.0072993">
<!-- The value of alpha_QED to be used for the spectrum. -->
</ISR_alpha>
<ISR_m_in type="float" value="0.000511">
<!-- The mass of the incoming particle. -->
</ISR_m_in>
<ISR_Q_max type="float" value="sqrts">
<!-- The hard scale which cuts off photon radiation. -->
</ISR_Q_max>
<ISR_LLA_order type="0/1/2/3" value="3">
<!-- The order of the leading-logarithmic approximation. -->
</ISR_LLA_order>
<ISR_map type="T/F" value="T">
<!-- Whether to use a mapping of the singularity at x=1 when evaluating the structure function (recommended; note that switching this off might even lead to an uncaught arithmetic exception). -->
</ISR_map>
<EPA_on type="T/F" value="F">
<!-- Whether to use the EPA spectrum (photon beam). -->
</EPA_on>
<EPA_map type="T/F" value="T">
<!-- Whether to apply a mapping to improve convergence. -->
</EPA_map>
<EPA_alpha type="float" value="0.0072993">
<!-- The value of alpha_QED to be used for the spectrum. -->
</EPA_alpha>
<EPA_m_in type="float" value="0.000511">
<!-- The mass of the incoming beam particle. -->
</EPA_m_in>
<EPA_mX type="float" value="4">
<!-- The lower cutoff for the produced invariant mass. -->
</EPA_mX>
<EPA_Q_max type="float" value="4">
<!-- The upper cutoff on the virtuality of the photon (Qmax&gt;0). -->
</EPA_Q_max>
<EPA_x0 type="float" value="0">
<!-- The lower cutoff on the energy fraction of the incoming photon -->
</EPA_x0>
<EPA_x1 type="float" value="0">
<!-- The upper cutoff on the energy fraction of the incoming photon -->
</EPA_x1>
</beam_input_2>
</whizard>
""" % modelparams)
    self.getInputFiles(model)

  def getInputFiles(self, model):
    """Get the proper input parameter file, usually LesHouches."""
    if 'process_input' not in self.paramdict:
      self.paramdict['process_input'] = {}
    if 'input_file' not in self.paramdict['process_input']:
      res = self.genmodel.getFile(model)
      if not res['OK']:
        self.paramdict['process_input']['input_file'] = ''
        self.paramdict['process_input']['input_slha_format'] = 'F'
      else:
        self.paramdict['process_input']['input_file'] = res['Value']
        self.paramdict['process_input']['input_slha_format'] = 'T'

  def modelParams(self, model):
    """Get the model parameters."""
    modelparams = []
    res = self.genmodel.getParamsForWhizard(model)
    if not res['OK']:
      return ""
    modelparams.append("<parameter_input>")

    modelparams.append(res['Value'])

    modelparams.append("</parameter_input>")

    return "\n".join(modelparams)

  def toXML(self, fname='whizard.xml'):
    """Write to XML."""
    tree = ElementTree(self.whizardxml)
    tree.write(fname)
    return S_OK()

  def getMainFields(self):
    """Get the main fields."""
    listoffields = []
    for elem in list(self.whizardxml):
      listoffields.append(elem.tag)
    return S_OK(listoffields)

  def getOptionsForField(self, field):
    """Get the options of a given field."""
    options = []
    element = self.whizardxml.find(field)
    if element is None:
      return S_ERROR("Field %s does not exist" % field)
    for subelements in list(element):
      options.append(subelements.tag)
    return S_OK(options)

  def getValue(self, field):
    """Get the value for a given field/option."""
    element = self.whizardxml.find(field)
    return S_OK(element.attrib['value'])

  def changeAndReturn(self, paramdict):
    """Update the options, and returns the modified XML object."""
    self.paramdict.update(paramdict)
    res = self.checkFields(self.paramdict)
    if not res['OK']:
      return res
    for key, val in self.paramdict.items():
      for subkey in val.keys():
        subelement = self.whizardxml.find(key + "/" + subkey)
        subelement.attrib['value'] = val[subkey]
    return S_OK(self.whizardxml)

  def getAsDict(self):
    """Get the content as dict, like the one used for setting the options."""
    whiz_opt = {}
    for element in list(self.whizardxml):
      whiz_opt[element.tag] = {}
      for item in list(element):
        val = item.attrib['value']
        if isinstance(val, six.string_types):
          val = val.rstrip()
        whiz_opt[element.tag][item.tag] = val
    return S_OK(whiz_opt)

  def checkFields(self, paramdict):
    """Make sure all supplied fields are exisiting somewhere."""
    for key, val in paramdict.items():
      element = self.whizardxml.find(key)
      if element is None:
        return S_ERROR("Element %s is not in the allowed parameters" % key)
      for subkey, value in val.items():
        subelement = self.whizardxml.find(key + "/" + subkey)
        if subelement is None:
          return S_ERROR("Key %s/%s is not in the allowed parameters" % (key, subkey))
        etype = subelement.attrib['type']
        if etype == 'float':
          if not isinstance(value, (float, int, int)) or isinstance(value, bool):
            return S_ERROR("%s should be a float" % (key + "/" + subkey))
        elif etype == 'T/F':
          if value != 'T' and value != 'F':
            return S_ERROR("%s should be either 'T' or 'F'" % (key + "/" + subkey))
        elif etype == 'integer' or etype == '0/1/2/3':
          if not isinstance(value, six.integer_types) or (etype == '0/1/2/3' and value not in list(range(4))):
            return S_ERROR("%s should be an integer" % (key + "/" + subkey))
        elif etype == 'string':
          if not isinstance(value, six.string_types):
            return S_ERROR("%s should be a string" % (key + "/" + subkey))
        elif etype == 'floatarray':
          error = False
          if not isinstance(value, six.string_types):
            error = True
          else:
            valelem = value.split()
            if len(valelem) < 2:
              error = True
          if error:
            return S_ERROR("%s should be a string with spaces, e.g. '0 1 2 3'" % (key + "/" + subkey))
    return S_OK()

  def toWhizardDotIn(self, fname):
    """Write the options to the whizard.in."""
    lines = []
    for elem in list(self.whizardxml):
      tag = elem.tag
      if tag.count("beam_input"):
        tag = "beam_input"
      lines.append("&%s" % tag)
      for subelem in list(elem):
        val = subelem.get('value')
        if val == 'sqrts':
          continue
        if subelem.get('type') == 'string':
          val = '"%s"' % val
        if val == '000':
          val = '0 0 0'
        if val == '0.0.0':
          val = '0.0 0.0'
        if val == '0..0..0':
          val = '\n 1 20000\n 10 20000\n 1 20000'
        lines.append(' %s = %s' % (subelem.tag, val))
      lines.append('/')
    with open(fname, "w") as of:
      of.write("\n".join(lines))
      of.write("\n")
    return S_OK(True)

  def fromWhizardDotIn(self, filename):
    """Given a whizard.in, create the parameter dict used for the production definition."""
    pdict = {}
    beam_input_idx = 0
    curkey = ""
    with open(filename, "r") as whizin:
      for line in whizin:
        line = line.rstrip().lstrip()
        if not line or line[0] in ["!", "/"]:
          continue

        if line[0] == "&":
          key = line.split("&")[1].rstrip().lstrip()
          if key.count("beam_input"):
            if not beam_input_idx:
              curkey = key + "_1"
              beam_input_idx += 1
            else:
              curkey = key + "_2"
          else:
            curkey = key
          pdict[curkey] = {}
        elif line.count("="):
          elems = line.split("=")
          key = elems[0].lstrip().rstrip()
          val = str("=".join(elems[1:])).lstrip().rstrip().lstrip('"').rstrip('"')
          if not val.count("T") and not val.count("F"):
            if val.count("."):
              try:
                val = float(val)
              except ValueError:
                pass
            else:
              try:
                val = int(val)
              except ValueError:
                pass
          pdict[curkey][key] = val
    return self.changeAndReturn(pdict)


def main():
  import sys
  import pprint
  fname = sys.argv[1]
  model = 'sm'
  if len(sys.argv) > 2:
    model = sys.argv[2]
  wh = WhizardOptions(model)
  res = wh.fromWhizardDotIn(fname)
  if not res['OK']:
    LOG.error(res['Message'])
    if res['Message'].count("parameter_input"):
      LOG.error("Maybe you are trying to set a mass that will be overwritten by a LesHouches file?")
    return 1

  pp = pprint.PrettyPrinter(indent=2)
  pp.pprint(wh.getAsDict()['Value'])
  return 0


if __name__ == "__main__":
  exit(main())



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Utility functions."""
from __future__ import absolute_import
from DIRAC import gLogger

LOG = gLogger.getSubLogger(__name__)


def toInt(number, cond=None):
  """Cast number parameter to an integer.

  >>> number = toInt("1000")

  :param number: the number to cast (number of events, number of jobs)
  :type number: str or int
  :param cond: unary function to check validity of number, range, etc.
  :return: The success or the failure of the casting
  :rtype: bool, int or None
  """
  if number is None:
    return number

  try:
    number = int(number)
  except ValueError:
    LOG.error("Argument is not an integer")
    return False
  if cond is not None and not cond(number):
    LOG.error("Argument does not pass condition: %r" % cond)
    return False
  return number


def listify(value, cast=None):
  """Turn a comma separate string into a list.

  :param str value: string to turn into a list
  :param cast: function to apply in all values, e.g., `int`
  :returns: list, empty elements are stripped
  """
  if isinstance(value, list):
    thisList = value
  else:
    thisList = [val.strip() for val in value.strip('[]').split(',') if val.strip()]
  if cast is not None:
    thisList = [cast(val) for val in thisList]
  return thisList


def canConvert(value, funType):
  """Check if value can be converted to a variable of type funType.

  >>> canConvert("5.",int)
  False

  >>> canConvert("3",int)
  True

  :param value: variable to be checked for conversion
  :param funType: function used for conversion e.g. int(), float()
  :return: True if can be converved to funType, else False
  :rtype: bool
  """
  try:
    funType(value)
    return True
  except ValueError:
    return False


def lowerFirst(inputString):
  """Return string with first charecter converted to lower case."""
  return inputString[0].lower() + inputString[1:]



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Download files directly from an SRM.

Use this to download files not available in the DiracFileCatalog
"""
from __future__ import absolute_import
from ILCDIRAC.Interfaces.API.NewInterface.LCUtilityApplication import LCUtilityApplication
from DIRAC import S_OK, S_ERROR, gLogger
from DIRAC.Core.Workflow.Parameter import Parameter

LOG = gLogger.getSubLogger(__name__)
__RCSID__ = "$Id$"


class GetSRMFile(LCUtilityApplication):
  """Gets a given file from storage directly using srm path.

  Usage:

  >>> gf = GetSRMFile()
  >>> fdict = {"file" : "srm://srm-public.cern.ch/castor/cern.ch/grid/ilc/prod/clic/1tev/Z_uds/gen/0/nobeam_nobrem_0-200.stdhep","site":"CERN-SRM"}
  >>> gf.setFiles(fdict)
  """

  def __init__(self, paramdict=None):
    self.files = {}
    super(GetSRMFile, self).__init__(paramdict)
    self._modulename = "GetSRMFile"
    self.appname = self._modulename
    self._moduledescription = "Module to get files directly from Storage"

  def setFiles(self, fdict):
    """Specify the files you need.

    :param fdict: file dictionary: {'file':'site'}, can be also [{},{}] etc.
    :type fdict: :class:`python:dict`, or :any:`python:list` of :class:`python:dict`
    """
    kwargs = {"fdict": fdict}
    if not isinstance(fdict, (dict, list)):
      return self._reportError('Expected dict or list of dicts for fdict', __name__, **kwargs)

    self.files = fdict

  def _applicationModule(self):
    m1 = self._createModuleDefinition()
    m1.addParameter(Parameter("srmfiles", [], "list", "", "", False, False, "list of files to retrieve"))
    m1.addParameter(Parameter("debug", False, "bool", "", "", False, False, "debug mode"))
    return m1

  def _applicationModuleValues(self, moduleinstance):
    moduleinstance.setValue("srmfiles", self.files)
    moduleinstance.setValue("debug", self.debug)

  def _userjobmodules(self, stepdefinition):
    res1 = self._setApplicationModuleAndParameters(stepdefinition)
    if not res1["OK"]:
      return S_ERROR("userjobmodules method failed")
    return S_OK()

  def _prodjobmodules(self, step):
    LOG.error("This application is not meant to be used in Production context")
    return S_ERROR('Should not use in Production')

  def _checkConsistency(self, job=None):

    if not self.files:
      return S_ERROR("The file list was not defined")

    if isinstance(self.files, dict):
      self.files = [self.files]

    # For the getInputFromApp to work, we nedd to tell the application about the expected OutputFile
    flist = ''
    for fdict in self.files:
      filePath = fdict['file']
      bname = filePath.split("/")[-1]
      flist += bname + ";"

    self.setOutputFile(flist.rstrip(";"))

    return S_OK()

  def _addParametersToStep(self, step):
    res = self._addBaseParameters(step)
    if not res["OK"]:
      return S_ERROR("Failed to set base parameters")
    return S_OK()



#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""
KKMC: Interface to the KKMCee generator

.. versionadded:: v29r2p5

Usage:

>>> kkmc = KKMC ()
>>> kkmc.setVersion('LCG_97a_FCC_4')
>>> kkmc.setConfigFile('__path_to__/process.input')

or

>>> kkmc = KKMC ()
>>> kkmc.setVersion('LCG_97a_FCC_4')
>>> kkmc.setEvtType('Mu')
>>> kkmc.setEnergy(91.2)
>>> kkmc.setNumberOfEvents(1000)
>>> kkmc.setOutputFile('kkmu_1000.LHE')

"""

from __future__ import absolute_import
import types
import os

from ILCDIRAC.Interfaces.API.NewInterface.LCApplication import LCApplication
from DIRAC import S_OK, S_ERROR, gLogger
from DIRAC.Core.Workflow.Parameter import Parameter
from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations


LOG = gLogger.getSubLogger(__name__)

__RCSID__ = "$Id$"


class KKMC(LCApplication):
  """KKMC Application Class."""

  def __init__(self, paramdict=None):
    self.randomSeed = -1
    self.eventType = ''
    self.kkmcConfigFile = ''
    self.seedFile = ''
    super(KKMC, self).__init__(paramdict)
    # Those 5 need to come after default constructor
    self._modulename = 'KKMCAnalysis'
    self._moduledescription = 'Module to run KKMC'
    self.appname = 'kkmc'
    self.datatype = 'GEN'
    self._ops = Operations()
    self._extension = 'lhe'

  def setEvtType(self, evttype):
    """ Optional: Flavour to be generated (Mu|Tau|UDS|C|B|Hadrons).

    :param str evttype: Flavour to be generated
    """
    self._checkArgs({'evttype': (str,)})
    if self.addedtojob:
      return self._reportError("Cannot modify this attribute once application has been added to Job")
    self.eventType = evttype
    return S_OK()

  def setConfigFile(self, kkmcConfigFilePath):
    """Set the KKMC options to be used.

    :param str kkmcConfigFilePath: Path to the KKMC input file.
    """
    self._checkArgs({'kkmcConfigFilePath': (str,)})

    # Chech if file exist
    if not os.path.isfile(kkmcConfigFilePath):
      return self._reportError('KKMC config file does not exist!')

    # Read file
    self.kkmcConfigFile = open(kkmcConfigFilePath).read()

    return None

  def setSeedFile(self, seedFilePath):
    """ Optional: set the file to be used for seeding (randomly generated, if missing)

    :param str seedFilePath: Path to the KKMC seed file.
    """
    self._checkArgs({'seedFilePath': (str,)})

    # Chech if file exist
    if not os.path.isfile(seedFilePath):
      return self._reportError('The seed file does not exist! %s' % seedFilePath)

    # Read file
    self.seedFile = open(seedFilePath).read()

    return None

  def _userjobmodules(self, stepdefinition):
    res1 = self._setApplicationModuleAndParameters(stepdefinition)
    res2 = self._setUserJobFinalization(stepdefinition)
    if not res1["OK"] or not res2["OK"]:
      return S_ERROR('userjobmodules failed')
    return S_OK()

  def _prodjobmodules(self, stepdefinition):
    res1 = self._setApplicationModuleAndParameters(stepdefinition)
    res2 = self._setOutputComputeDataList(stepdefinition)
    if not res1["OK"] or not res2["OK"]:
      return S_ERROR('prodjobmodules failed')
    return S_OK()

  def _checkConsistency(self, job=None):
    """Check consistency of the KKMC application, this is called from the `Job` instance.

    :param job: The instance of the job
    :type job: ~ILCDIRAC.Interfaces.API.NewInterface.Job.Job
    :returns: S_OK/S_ERROR
    """

    if not self.version:
      return S_ERROR('No version found!')

    if self.kkmcConfigFile:
      return S_OK()

    if self._jobtype != 'User':
      self._listofoutput.append({"outputFile": "@{OutputFile}", "outputPath": "@{OutputPath}",
                                 "outputDataSE": '@{OutputSE}'})
      return S_OK()

    if not self.eventType and not self.energy and not self.numberOfEvents and not self.outputFile:
      return S_ERROR('No config file set!')
    if not self.eventType:
      return S_ERROR('No event type set!')
    if not self.energy:
      return S_ERROR('No energy set!')
    if not self.numberOfEvents:
      return S_ERROR('No number of events set!')
    if not self.outputFile:
      return S_ERROR('No output file set!')

    return S_OK()

  def _applicationModule(self):
    md1 = self._createModuleDefinition()
    md1.addParameter(Parameter("debug", False, "bool", "", "", False, False, "debug mode"))
    md1.addParameter(Parameter("kkmcConfigFile", '', "string", "", "", False, False, "KKMC steering options"))
    md1.addParameter(Parameter("seedFile", '', "string", "", "", False, False, "Seed file for the generator"))
    md1.addParameter(
        Parameter(
            "eventType",
            '',
            "string",
            "",
            "",
            False,
            False,
            "Flavour to be generated (Mu|Tau|UDS|C|B|Hadrons)"))
    return md1

  def _applicationModuleValues(self, moduleinstance):
    moduleinstance.setValue("debug", self.debug)
    moduleinstance.setValue("kkmcConfigFile", self.kkmcConfigFile)
    moduleinstance.setValue("seedFile", self.seedFile)
    moduleinstance.setValue("eventType", self.eventType)

  def _checkWorkflowConsistency(self):
    return self._checkRequiredApp()



#!/usr/bin/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test the InputDataResolution class."""

from __future__ import absolute_import
import sys
import unittest
from mock import patch, MagicMock as Mock

from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertDiracFailsWith, \
    assertDiracSucceeds, assertMockCalls
from DIRAC import S_OK, S_ERROR

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Core.Utilities.InputDataResolution'

# pylint: disable=too-many-public-methods


class TestInputDataResolution(unittest.TestCase):
  """Test the different methods of the class."""

  def setUp(self):
    self.ops_mock = Mock()
    self.mfac_mock = Mock()
    mocked_modules = {'DIRAC.ConfigurationSystem.Client.Helpers.Operations': self.ops_mock,
                       'DIRAC.Core.Utilities.ModuleFactory': self.mfac_mock}
    self.module_patcher = patch.dict(sys.modules, mocked_modules)
    self.module_patcher.start()
    self.idr = None

  def tearDown(self):
    self.module_patcher.stop()

  def test_execute_input_missing_fail(self):
    module_mock = Mock()
    module_mock.execute.return_value = {'OK': True, 'Failed': ['myReplicaSite1', 'other_failure'],
                                         'Successful': {'WorkingSite': True}}
    self.mfac_mock.ModuleFactory().getModule.return_value = S_OK(module_mock)
    from ILCDIRAC.Core.Utilities.InputDataResolution import InputDataResolution
    self.idr = InputDataResolution({'Job': {'InputDataPolicy': ['/myTestPolicy/some/path']},
                                      'Configuration': {'SiteName': 'myTestSitename'}})
    assertDiracFailsWith(self.idr.execute(), 'failed to access all of requested input data', self)
    self.mfac_mock.ModuleFactory().getModule.assert_called_once_with(
        '/myTestPolicy/some/path',
        {'Job': {'InputDataPolicy': ['/myTestPolicy/some/path']},
         'Configuration': {'SiteName': 'myTestSitename'}})

  def test_execute_resolveinput_fails(self):
    from ILCDIRAC.Core.Utilities.InputDataResolution import InputDataResolution
    self.idr = InputDataResolution({'Configuration': {'SiteName': 'myTestSitename'}})
    ops_mock = Mock()
    ops_mock.getOptionsDict.return_value = None
    self.idr.ops = ops_mock
    assertDiracFailsWith(self.idr.execute(), 'Could not resolve InputDataPolicy from /InputDataPolicy', self)
    ops_mock.getOptionsDict.assert_called_once_with('/InputDataPolicy')

  def test_execute_runmodule_fails(self):
    self.mfac_mock.ModuleFactory().getModule.return_value = S_ERROR('module_test_mockerr')
    from ILCDIRAC.Core.Utilities.InputDataResolution import InputDataResolution
    self.idr = InputDataResolution({'Configuration': {},
                                      'Job': {'InputDataPolicy': '/myTestPolicy/some/path'},
                                      'IgnoreMissing': False})
    with patch('%s.DIRAC.siteName' % MODULE_NAME, new=Mock(return_value='SiteNameTestdirac')):
      assertDiracFailsWith(self.idr.execute(), 'module_test_mockerr', self)

  def test_execute_resolveinput_none_successful(self):
    module_mock = Mock()
    module_mock.execute.return_value = {'OK': True, 'Failed': ['myReplicaSite1', 'other_failure'],
                                         'Successful': {}}
    self.mfac_mock.ModuleFactory().getModule.return_value = S_OK(module_mock)
    from ILCDIRAC.Core.Utilities.InputDataResolution import InputDataResolution
    self.idr = InputDataResolution({'Job': {'InputDataPolicy': ['/myTestPolicy/some/path']},
                                      'Configuration': {'SiteName': 'myTestSitename'},
                                      'IgnoreMissing': True})
    assertDiracFailsWith(self.idr.execute(), 'could not access any requested input data', self)

  def test_execute_ignoremissing_works(self):
    module_mock = Mock()
    module_mock.execute.side_effect = [{'OK': True, 'Failed': ['myReplicaSite1', 'other_failure'],
                                          'Successful': {'WorkingSite1': True, 'OtherGoodSite': True}},
                                        {'OK': True, 'Failed': [],
                                          'Successful': {'other_site': True, 'TestSite1': True}}]
    self.mfac_mock.ModuleFactory().getModule.return_value = S_OK(module_mock)
    from ILCDIRAC.Core.Utilities.InputDataResolution import InputDataResolution
    self.idr = InputDataResolution({'Job': {'InputDataPolicy': ['/myTestPolicy/some/path',
                                                                      'other_policy/path']},
                                      'Configuration': {'SiteName': 'myTestSitename'},
                                      'IgnoreMissing': True})
    res = self.idr.execute()
    assertDiracSucceeds(res, self)
    assertEqualsImproved(res, {'OK': True, 'Value': None, 'Failed': [],
                                 'Successful': {'WorkingSite1': True, 'OtherGoodSite': True,
                                                  'other_site': True, 'TestSite1': True}}, self)
    assertMockCalls(self.mfac_mock.ModuleFactory().getModule, [
        ('/myTestPolicy/some/path', {'Job': {'InputDataPolicy': ['/myTestPolicy/some/path',
                                                                     'other_policy/path']},
                                     'Configuration': {'SiteName': 'myTestSitename'},
                                     'IgnoreMissing': True}),
        ('other_policy/path', {'Job': {'InputDataPolicy': ['/myTestPolicy/some/path', 'other_policy/path']},
                               'Configuration': {'SiteName': 'myTestSitename'}, 'IgnoreMissing': True})],
        self)

  def test_execute_allworks(self):
    module_mock = Mock()
    module_mock.execute.side_effect = [{'OK': True, 'Failed': {},
                                          'Successful': {'some_site': True, 'OtherGoodSite': True}}]
    self.mfac_mock.ModuleFactory().getModule.return_value = S_OK(module_mock)
    from ILCDIRAC.Core.Utilities.InputDataResolution import InputDataResolution
    self.idr = InputDataResolution({'Configuration': {'SiteName': 'myTestSitename'},
                                      'IgnoreMissing': True})
    ops_mock = Mock()
    ops_mock.getOptionsDict.return_value = {'Value': {'myTestSitename':
                                                         ' module_path1,other_modpath     , lastmodule'}}
    self.idr.ops = ops_mock
    res = self.idr.execute()
    assertDiracSucceeds(res, self)
    assertEqualsImproved(res, {'OK': True, 'Value': None, 'Failed': {},
                                 'Successful': {'some_site': True, 'OtherGoodSite': True}}, self)
    self.mfac_mock.ModuleFactory().getModule.assert_called_once_with(
        'module_path1', {'Job': {}, 'Configuration': {'SiteName': 'myTestSitename'}, 'IgnoreMissing': True})

  def test_execute_usedefaultoptions(self):
    module_mock = Mock()
    module_mock.execute.side_effect = [{'OK': True, 'Successful': {'some_site': True,
                                                                        'OtherGoodSite': True}}]
    self.mfac_mock.ModuleFactory().getModule.return_value = S_OK(module_mock)
    from ILCDIRAC.Core.Utilities.InputDataResolution import InputDataResolution
    self.idr = InputDataResolution({'Configuration': {'SiteName': 'myTestSitename'},
                                      'IgnoreMissing': True})
    ops_mock = Mock()
    ops_mock.getOptionsDict.return_value = {'Value': {'wrong_sitename':
                                                         ' module_path1,other_modpath     , lastmodule',
                                                         'Default': 'my/_path/module/ ,  dontusethis      '}}
    self.idr.ops = ops_mock
    res = self.idr.execute()
    assertDiracSucceeds(res, self)
    assertEqualsImproved(res, {'OK': True, 'Value': None, 'Failed': [],
                                 'Successful': {'some_site': True, 'OtherGoodSite': True}}, self)
    self.mfac_mock.ModuleFactory().getModule.assert_called_once_with(
        'my/_path/module/', {'Job': {}, 'Configuration': {'SiteName': 'myTestSitename'},
                             'IgnoreMissing': True})

  def test_execute_nopolicies(self):
    module_mock = Mock()
    module_mock.execute.side_effect = [{'OK': True, 'Successful': {'some_site': True,
                                                                        'OtherGoodSite': True}}]
    self.mfac_mock.ModuleFactory().getModule.return_value = S_OK(module_mock)
    from ILCDIRAC.Core.Utilities.InputDataResolution import InputDataResolution
    self.idr = InputDataResolution({'Configuration': {'SiteName': 'myTestSitename'},
                                      'IgnoreMissing': True})
    ops_mock = Mock()
    ops_mock.getOptionsDict.return_value = {'Value': {'wrong_sitename':
                                                         ' module_path1,other_modpath     , lastmodule',
                                                         'NotDefault': 'my/_path/module/ ,'}}
    self.idr.ops = ops_mock
    assertDiracFailsWith(self.idr.execute(), 'could not access any requested input data', self)
    self.assertFalse(self.mfac_mock.ModuleFactory().getModule.called)



#!/usr/bin/env python
#
# Copyright (c) 2009-2022 CERN. All rights nots expressly granted are
# reserved.
#
# This file is part of iLCDirac
# (see ilcdirac.cern.ch, contact: ilcdirac-support@cern.ch).
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# In applying this licence, CERN does not waive the privileges and
# immunities granted to it by virtue of its status as an
# Intergovernmental Organization or submit itself to any jurisdiction.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
"""Test the WhizardOption class."""

from __future__ import absolute_import
import sys
import unittest
from xml.etree.ElementTree import fromstring
from mock import mock_open, patch, MagicMock as Mock
import six

from ILCDIRAC.Core.Utilities.WhizardOptions import WhizardOptions, getDict, main
from Tests.Utilities.GeneralUtils import assertEqualsImproved, assertDiracFailsWith, \
    assertDiracSucceeds, assertDiracSucceedsWith_equals, assertEqualsXmlTree, assertMockCalls
from Tests.Utilities.FileUtils import FileUtil

__RCSID__ = "$Id$"

MODULE_NAME = 'ILCDIRAC.Core.Utilities.WhizardOptions'
BUILTIN_NAME = 'builtins' if six.PY3 else '__builtin__'


class TestWhizardOptions(unittest.TestCase):  # pylint: disable=too-many-public-methods
  """Test the different methods of the class."""

  def setUp(self):
    self.whop = WhizardOptions()

  def test_getdict(self):
    result = getDict()
    assertDiracSucceedsWith_equals(result, {'process_input': {}, 'integration_input': {},
                                              'simulation_input': {}, 'diagnostics_input': {},
                                              'beam_input_1': {}, 'beam_input_2': {}}, self)

  def test_getinputfiles(self):
    self.whop.paramdict = getDict()['Value']
    self.whop.paramdict['process_input']['input_file'] = 'mytestfile.input'
    old_state = self.whop.paramdict.copy()
    self.assertIsNone(self.whop.getInputFiles('model'))
    assertEqualsImproved(self.whop.paramdict, old_state, self)

  def test_toxml(self):
    with patch('%s.open' % BUILTIN_NAME, mock_open()) as open_mock:
      assertDiracSucceeds(self.whop.toXML('mytestOutputFile.xml'), self)
      if six.PY3:
        open_mock.assert_any_call('mytestOutputFile.xml', 'w', encoding='us-ascii', errors='xmlcharrefreplace')
      else:
        open_mock.assert_any_call('mytestOutputFile.xml', 'wb')
      self.assertTrue(len(open_mock().write.mock_calls) > 30)

  def test_getmainfields(self):
    result = self.whop.getMainFields()
    assertDiracSucceedsWith_equals(result, ['process_input', 'integration_input', 'simulation_input',
                                              'diagnostics_input', 'parameter_input', 'beam_input_1',
                                              'beam_input_2'], self)

  def test_getoptionsforfield(self):
    assertDiracSucceedsWith_equals(self.whop.getOptionsForField('process_input'),
                                    ['process_id', 'cm_frame', 'sqrts', 'luminosity', 'polarized_beams',
                                      'structured_beams', 'beam_recoil', 'recoil_conserve_momentum',
                                      'filename', 'directory', 'input_file', 'input_slha_format'], self)
    assertDiracSucceedsWith_equals(self.whop.getOptionsForField('parameter_input'),
                                    ['GF', 'mZ', 'mW', 'mH', 'alphas', 'me', 'mmu', 'mtau', 'ms', 'mc', 'mb',
                                     'mtop', 'wtop', 'wZ', 'wW', 'wH', 'vckm11', 'vckm12', 'vckm13',
                                     'vckm21', 'vckm22', 'vckm23', 'vckm31', 'vckm32', 'vckm33', 'khgaz',
                                     'khgaga', 'khgg'], self)

  def test_getoptionsforfield_fails(self):
    assertDiracFailsWith(self.whop.getOptionsForField('testFieldDoesntExist'),
                          'field testfielddoesntexist does not exist', self)

  def test_whgetasdict(self):
    expected_dict = {
        'parameter_input': {
            'vckm21': '-0.2271',
            'vckm23': '0.04221',
            'vckm22': '0.97296',
            'mtop': '174',
            'mtau': '1.777',
            'vckm33': '0.99910',
            'khgaz': '1.000',
            'khgg': '1.000',
            'mZ': '91.1882',
            'wtop': '1.523',
            'mmu': '0.1066',
            'mH': '120',
            'khgaga': '1.000',
            'vckm32': '-0.04161',
            'GF': '1.16639E-5',
            'mW': '80.419',
            'vckm31': '0.00814',
            'vckm11': '0.97383',
            'vckm12': '0.2272',
            'vckm13': '0.00396',
            'me': '0.',
            'alphas': '0.1178',
            'mc': '0.54',
            'mb': '2.9',
            'wH': '0.3605E-02',
            'wW': '2.049',
            'ms': '0.',
            'wZ': '2.443'},
        'integration_input': {
            'exchange_lines': '3',
            'max_bins': '20',
            'read_cuts_file': '',
            'write_grids': 'T',
            'min_bins': '3',
            'threshold_mass': '-10',
            'double_off_shell_branchings': 'T',
            'extra_off_shell_lines': '1',
            'phase_space_only': 'F',
            'use_efficiency': 'F',
            'seed': '',
            'reset_seed_each_process': 'F',
            'accuracy_goal': '0',
            'default_energy_cut': '10',
            'single_off_shell_decays': 'T',
            'write_all_grids_file': '',
            'default_jet_cut': '10',
            'massive_fsr': 'T',
            'write_phase_space_channels_file': '',
            'write_all_grids': 'F',
            'weights_power': '0.25',
            'default_mass_cut': '4',
            'read_grids_force': 'T',
            'write_default_cuts_file': '',
            'time_limit_adaptation': '0',
            'efficiency_goal': '100',
            'splitting_depth': '1',
            'azimuthal_dependence': 'F',
            'read_model_file': '',
            'double_off_shell_decays': 'F',
            'generate_phase_space': 'T',
            'read_grids': 'F',
            'write_phase_space_file': '',
            'read_phase_space': 'T',
            'write_grids_file': '',
            'threshold_mass_t': '-10',
            'read_phase_space_file': '',
            'min_calls_per_channel': '0',
            'read_grids_file': '',
            'calls': '1 50000 10 50000 1 1500000',
            'use_equivalences': 'T',
            'single_off_shell_branchings': 'T',
            'show_deleted_channels': 'F',
            'write_grids_raw': 'F',
            'user_cut_mode': '0',
            'min_calls_per_bin': '10',
            'stratified': 'T',
            'off_shell_lines': '1',
            'user_weight_mode': '0',
            'read_grids_raw': 'F',
            'default_q_cut': '4'},
        'simulation_input': {
            'read_events': 'F',
            'min_file_count': '1',
            'shower_t_min': '1.0',
            'unweighted': 'T',
            'keep_beam_remnants': 'T',
            'recalculate': 'F',
            'shower_md': '0.330',
            'shower_mc': '1.5',
            'shower_mb': '4.8',
            'write_events_raw_file': '',
            'n_calls': '0',
            'fragmentation_method': '3',
            'shower_alpha_s': '0.2',
            'fragment': 'T',
            'shower_mu': '0.330',
            'shower_ms': '0.500',
            'write_events_file': '',
            'write_events': 'T',
            'bytes_per_file': '0',
            'n_events_warmup': '0',
            'write_events_raw': 'F',
            'safety_factor': '1',
            'guess_color_flow': 'F',
            'shower_running_alpha_s': 'F',
            'keep_initials': 'T',
            'pythia_parameters': 'PMAS(25,1)=120.; PMAS(25,2)=0.3605E-02; MSTU(22)=20 ;MSTJ(28)=2 ;PARJ(21)=0.40000;PARJ(41)=0.11000; PARJ(42)=0.52000; PARJ(81)=0.25000; PARJ(82)=1.90000; MSTJ(11)=3; PARJ(54)=-0.03100; PARJ(55)=-0.00200;PARJ(1)=0.08500; PARJ(3)=0.45000; PARJ(4)=0.02500; PARJ(2)=0.31000; PARJ(11)=0.60000; PARJ(12)=0.40000; PARJ(13)=0.72000;PARJ(14)=0.43000; PARJ(15)=0.08000; PARJ(16)=0.08000; PARJ(17)=0.17000; MSTP(3)=1;',
            'read_events_force': 'T',
            'write_events_format': '20',
            'events_per_file': '5000000',
            'shower_nf': '5',
            'pythia_processes': '',
            'read_events_raw_file': '',
            'user_fragmentation_mode': '0',
            'normalize_weight': 'T',
            'max_file_count': '999',
            'write_weights_file': '',
            'shower': 'F',
            'n_events': '0',
            'shower_lambda': '0.29',
            'write_weights': 'F'},
        'diagnostics_input': {
            'show_event': 'F',
            'time_limit': '0',
            'screen_diagnostics': 'F',
            'show_histograms': 'F',
            'screen_events': 'F',
            'screen_histograms': 'F',
            'plot_width': '130',
            'plot_height': '90',
            'show_overflow': 'F',
            'show_cuts': 'T',
            'plot_history': 'T',
            'slha_ignore_errors': 'F',
            'show_excess': 'T',
            'show_results': 'T',
            'show_history': 'T',
            'show_pythia_statistics': 'T',
            'warn_empty_channel': 'F',
            'catch_signals': 'T',
            'write_logfile_file': '',
            'show_phase_space': 'F',
            'write_logfile': 'T',
            'slha_rewrite_input': 'T',
            'show_histories': 'F',
            'chattiness': '4',
            'show_pythia_initialization': 'T',
            'plot_grids_channels': '',
            'plot_grids_logscale': '10',
            'show_input': 'T',
            'read_analysis_file': '',
            'show_pythia_banner': 'T',
            'plot_excess': 'T',
            'show_weights': 'T'},
        'process_input': {
            'polarized_beams': 'T',
            'recoil_conserve_momentum': 'F',
            'input_file': '',
            'sqrts': '3000',
            'luminosity': '0',
            'input_slha_format': 'F',
            'filename': 'whizard',
            'structured_beams': 'T',
            'process_id': '',
            'cm_frame': 'T',
            'directory': '',
            'beam_recoil': 'F'},
        'beam_input_1': {
            'ISR_LLA_order': '3',
            'energy': '0',
            'EPA_on': 'F',
            'particle_code': '0',
            'ISR_Q_max': 'sqrts',
            'angle': '0',
            'ISR_on': 'T',
            'polarization': '0.0 0.0',
            'EPA_alpha': '0.0072993',
            'ISR_map': 'T',
            'particle_name': 'e1',
            'USER_spectrum_on': 'T',
            'EPA_x1': '0',
            'EPA_x0': '0',
            'direction': '0 0 0',
            'ISR_alpha': '0.0072993',
            'EPA_Q_max': '4',
            'ISR_m_in': '0.000511',
            'USER_spectrum_mode': '11',
            'EPA_m_in': '0.000511',
            'vector_polarization': 'F',
            'EPA_map': 'T',
            'EPA_mX': '4'},
        'beam_input_2': {
            'ISR_LLA_order': '3',
            'energy': '0',
            'EPA_on': 'F',
            'particle_code': '0',
            'ISR_Q_max': 'sqrts',
            'angle': '0',
            'ISR_on': 'T',
            'polarization': '0.0 0.0',
            'EPA_alpha': '0.0072993',
            'ISR_map': 'T',
            'particle_name': 'E1',
            'USER_spectrum_on': 'T',
            'EPA_x1': '0',
            'EPA_x0': '0',
            'direction': '0 0 0',
            'ISR_alpha': '0.0072993',
            'EPA_Q_max': '4',
            'ISR_m_in': '0.000511',
            'USER_spectrum_mode': '-11',
            'EPA_m_in': '0.000511',
            'vector_polarization': 'F',
            'EPA_map': 'T',
            'EPA_mX': '4'}}
    result = self.whop.getAsDict()
    assertDiracSucceedsWith_equals(result, expected_dict, self)

  def test_checkfields(self):
    self.whop.whizardxml = fromstring(
        '<whizard> <process_input> <mytestelement type="floatarray"> </mytestelement> <test_bool type="T/F"> </test_bool> <test_bool2 type="T/F"> </test_bool2> <test_integer type="integer"> </test_integer> </process_input> <test_superelem> <test_float type="float"> </test_float> <test_float2 type="float"> </test_float2> <test_string type="string"> </test_string> <test_ignoreme type="invalid_type"></test_ignoreme> </test_superelem> </whizard>')
    assertDiracSucceeds(self.whop.checkFields(
        {'process_input': {'mytestelement': '842 021.2 123 98724 81', 'test_bool': 'T', 'test_bool2': 'F',
                            'test_integer': 8492},
         'test_superelem': {'test_float': 824.2, 'test_float2': 98421, 'test_string': 'oijrsg'}
      }), self)

  def test_checkfields_missing_field(self):
    assertDiracFailsWith(self.whop.checkFields(
        {'parameter_input': {}, 'nonexistent_element_findandtestme': True}),
        'Element nonexistent_element_findandtestme is not in the allowed parameters', self)

  def test_checkfields_missing_subfield(self):
    assertDiracFailsWith(self.whop.checkFields(
        {'parameter_input': {'nonexistent_element_findandtestme': True}}),
        'key parameter_input/nonexistent_element_findandtestme is not in the allowed parameters',
        self)

  def test_checkfields_fake_float(self):
    self.whop.whizardxml = fromstring(
        '<whizard> <process_input> <mytestelement type="float"> </mytestelement> </process_input> </whizard>')
    assertDiracFailsWith(self.whop.checkFields(
        {'process_input': {'mytestelement': True}}), 'process_input/mytestelement should be a float',
        self)

  def test_checkfields_fake_bool(self):
    self.whop.whizardxml = fromstring(
        '<whizard> <process_input> <mytestelement type="T/F"> </mytestelement> </process_input> </whizard>')
    assertDiracFailsWith(self.whop.checkFields(
        {'process_input': {'mytestelement': 125311}}),
        "process_input/mytestelement should be either 'T' or 'F'", self)

  def test_checkfields_fake_integer(self):
    self.whop.whizardxml = fromstring(
        '<whizard> <process_input> <mytestelement type="integer"> </mytestelement> </process_input> </whizard>')
    assertDiracFailsWith(self.whop.checkFields(
        {'process_input': {'mytestelement': 'mystring'}}),
        'process_input/mytestelement should be an integer', self)

  def test_checkfields_fake_string(self):
    self.whop.whizardxml = fromstring(
        '<whizard> <process_input> <mytestelement type="string"> </mytestelement> </process_input> </whizard>')
    assertDiracFailsWith(self.whop.checkFields(
        {'process_input': {'mytestelement': True}}),
        'process_input/mytestelement should be a string', self)

  def test_checkfields_fake_floatarray(self):
    self.whop.whizardxml = fromstring(
        '<whizard> <process_input> <mytestelement type="floatarray"> </mytestelement> </process_input> </whizard>')
    assertDiracFailsWith(self.whop.checkFields(
        {'process_input': {'mytestelement': 892354.3}}),
        'process_input/mytestelement should be a string with spaces', self)

  def test_checkfields_empty_floatarray(self):
    self.whop.whizardxml = fromstring(
        '<whizard> <process_input> <mytestelement type="floatarray"> </mytestelement> </process_input> </whizard>')
    assertDiracFailsWith(self.whop.checkFields(
        {'process_input': {'mytestelement': ''}}),
        'process_input/mytestelement should be a string with spaces', self)

  def test_checkfields_floatarray_len1(self):
    self.whop.whizardxml = fromstring(
        '<whizard> <process_input> <mytestelement type="floatarray"> </mytestelement> </process_input> </whizard>')
    assertDiracFailsWith(self.whop.checkFields(
        {'process_input': {'mytestelement': '123'}}),
        'process_input/mytestelement should be a string with spaces', self)

  def test_towhizarddotin(self):
    self.whop.whizardxml = fromstring(
        '<whizard> <process_input> <mytestelement type="floatarray" value="sqrts"> </mytestelement> <test_bool type="T/F" value="000"> </test_bool> <test_bool2 type="T/F" value="0.0.0"> </test_bool2> <test_integer type="integer" value="0..0..0"> </test_integer> </process_input> <lalabeam_input1238> <test_float type="float" value="dontchangeanything"> </test_float> <test_string type="string" value="some_teststring"> </test_string> <test_noval></test_noval> </lalabeam_input1238> </whizard>')
    with patch('%s.open' % MODULE_NAME, mock_open()) as open_mock:
      result = self.whop.toWhizardDotIn('mytestfile.xml')
      assertDiracSucceeds(result, self)
      open_mock.assert_any_call('mytestfile.xml', 'w')
      assertMockCalls(
          open_mock().write, [
              '&process_input\n test_bool = 0 0 0\n test_bool2 = 0.0 0.0\n test_integer = \n 1 20000\n 10 20000\n 1 20000\n/\n&beam_input\n test_float = dontchangeanything\n test_string = "some_teststring"\n test_noval = None\n/', '\n'], self)

  def test_fromwhizarddotin(self):
    self.whop.paramdict = {}
    self.whop.whizardxml = fromstring(
        '<whizardxml> <beam_input_1> <beam_intensity type="float"> </beam_intensity> <beam_density type="0/1/2/3"> </beam_density> </beam_input_1> <beam_input_2> <FcoolTtestkey type="string"> </FcoolTtestkey> </beam_input_2> <myotherkey> </myotherkey> <testkey> <testcurkey type="string"> </testcurkey> </testkey> <lasttestkey> <myfloatarr type="floatarray"> </myfloatarr> <lasttestvalue type="T/F"> </lasttestvalue> </lasttestkey> </whizardxml>')
    expected_tree = fromstring(
        '<whizardxml> <beam_input_1> <beam_intensity type="float" value="84.2"> </beam_intensity> <beam_density type="0/1/2/3" value="0"> </beam_density> </beam_input_1> <beam_input_2> <FcoolTtestkey type="string" value="TmyFstring"> </FcoolTtestkey> </beam_input_2> <myotherkey> </myotherkey> <testkey> <testcurkey type="string" value="teststring_dontloseme."> </testcurkey> </testkey> <lasttestkey> <myfloatarr type="floatarray" value="943 0.1 01.2 9024.4"> </myfloatarr> <lasttestvalue type="T/F" value="F"> </lasttestvalue> </lasttestkey> </whizardxml>')
    expected_tree.find('beam_input_1/beam_intensity').attrib['value'] = 84.2
    expected_tree.find('beam_input_1/beam_density').attrib['value'] = 0
    file_contents = [['     !initial Comment, ignore this line   ', '/  Other comment.               ',
                        '                                 ', 'ignoreThisLineTOO', '&beam_input',
                        'beam_intensity=84.2', 'beam_density=0', '&myotherkey', '&testkey',
                        'testcurkey="teststring_dontloseme."', '&beam_input  ',
                        '  FcoolTtestkey  =  "TmyFstring"   ', '&lasttestkey',
                        'myfloatarr= 943 0.1 01.2 9024.4', 'lasttestvalue=F']]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('%s.open' % MODULE_NAME, mock_open(), create=True) as mo:
      mo.side_effect = (h for h in handles)
      result = self.whop.fromWhizardDotIn('filename.txt')
      assertDiracSucceeds(result, self)
      assertEqualsXmlTree(result['Value'], expected_tree, self)

  def test_main(self):
    print_mock = Mock()
    pprint_mock = Mock(return_value=print_mock)
    file_contents = [['     !initial Comment, ignore this line   ', '/  Other comment.               ',
                        '                                 ', 'ignoreThisLineTOO', '&process_input',
                        'process_id=myprocessid', 'cm_frame=F', '&integration_input', '&simulation_input',
                        'unweighted=F', '&diagnostics_input  ',
                        '  write_logfile_file  =  "my_diagnostics_logfile.txt"   ']]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('sys.argv', ['scriptname', 'filename.xml']), \
         patch('%s.open' % MODULE_NAME, mock_open(), create=True) as mo, \
         patch.dict(sys.modules, {'pprint': pprint_mock}):
      mo.side_effect = (h for h in handles)
      assertEqualsImproved(main(), 0, self)
      mo.assert_called_once_with('filename.xml', 'r')
      # if not running_on_docker():
      #  xml_dict = pprint_mock.PrettyPrinter.return_value.pprint.call_args[0][0]
      #  assertEqualsImproved( xml_dict[ 'process_input' ][ 'process_id' ], 'myprocessid', self )
      #  assertEqualsImproved( xml_dict[ 'process_input' ][ 'cm_frame' ], 'F', self )
      #  assertEqualsImproved( xml_dict[ 'simulation_input' ][ 'unweighted' ], 'F', self )
      #  assertEqualsImproved( xml_dict[ 'diagnostics_input' ][ 'write_logfile_file' ],
      #                        'my_diagnostics_logfile.txt', self )

  def test_main_other_model(self):
    print_mock = Mock()
    pprint_mock = Mock(return_value=print_mock)
    file_contents = [['     !initial Comment, ignore this line   ', '/  Other comment.               ',
                        '                                 ', 'ignoreThisLineTOO', '&process_input',
                        'process_id=myprocessid', 'cm_frame=F', '&integration_input', '&simulation_input',
                        'unweighted=F', '&diagnostics_input  ',
                        '  write_logfile_file  =  "my_diagnostics_logfile.txt"   ']]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('sys.argv', ['scriptname', 'filename.xml', 'mymodel']), \
         patch('%s.open' % MODULE_NAME, mock_open(), create=True) as mo, \
         patch.dict(sys.modules, {'pprint': pprint_mock}):
      mo.side_effect = (h for h in handles)
      assertEqualsImproved(main(), 0, self)
      mo.assert_called_once_with('filename.xml', 'r')
      # if not running_on_docker():
      #  xml_dict = pprint_mock.PrettyPrinter.return_value.pprint.call_args[0][0]
      #  assertEqualsImproved( xml_dict[ 'process_input' ][ 'process_id' ], 'myprocessid', self )
      #  assertEqualsImproved( xml_dict[ 'process_input' ][ 'cm_frame' ], 'F', self )
      #  assertEqualsImproved( xml_dict[ 'simulation_input' ][ 'unweighted' ], 'F', self )
      #  assertEqualsImproved( xml_dict[ 'diagnostics_input' ][ 'write_logfile_file' ],
      #                        'my_diagnostics_logfile.txt', self )

  def test_main_wrong_type(self):
    print_mock = Mock()
    pprint_mock = Mock(return_value=print_mock)
    file_contents = [['&process_input', 'process_id=myprocessid', 'cm_frame=02.2', '&integration_input',
                        '&simulation_input', 'unweighted=8924', '&diagnostics_input  ',
                        '  write_logfile_file  =  "my_diagnostics_logfile.txt"   ']]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('sys.argv', ['scriptname', 'filename.xml', 'mymodel']), \
         patch('%s.open' % MODULE_NAME, mock_open(), create=True) as mo, \
         patch.dict(sys.modules, {'pprint': pprint_mock}):
      mo.side_effect = (h for h in handles)
      assertEqualsImproved(main(), 1, self)
      mo.assert_called_once_with('filename.xml', 'r')

  def test_main_wrong_paraminput(self):
    print_mock = Mock()
    pprint_mock = Mock(return_value=print_mock)
    file_contents = [['&process_input', 'parameter_input=123', 'process_id=myprocessid', 'cm_frame=F',
                        '&integration_input', '&simulation_input', 'unweighted=F', '&diagnostics_input  ',
                        '  write_logfile_file  =  "my_diagnostics_logfile.txt"   ']]
    handles = FileUtil.getMultipleReadHandles(file_contents)
    with patch('sys.argv', ['scriptname', 'filename.xml', 'mymodel']), \
         patch('%s.open' % MODULE_NAME, mock_open(), create=True) as mo, \
         patch.dict(sys.modules, {'pprint': pprint_mock}):
      mo.side_effect = (h for h in handles)
      assertEqualsImproved(main(), 1, self)
      mo.assert_called_once_with('filename.xml', 'r')


class TestWhOptCustomTree(unittest.TestCase):
  """Test critical methods with a custom xml tree."""

  DEFAULT_TREE = '<mytree>    <testchild1 type="asd" value="10">        <testoption type="test" value="123"></testoption>        <other_option></other_option>        <lastoneIpromise></lastoneIpromise>    </testchild1>    <dontforgetme></dontforgetme>    <onlychild type="floatarray" value="asd" some_other_field="test"></onlychild></mytree>'

  def setUp(self):
    self.whop = WhizardOptions()
    self.whop.paramdict = {'testchild1': {'testoption': '123', 'other_option': None,
                                             'lastoneIpromise': None}, 'dontforgetme': {},
                            'onlychild': {}}
    self.whop.whizardxml = fromstring(TestWhOptCustomTree.DEFAULT_TREE)

  def test_getoptions_nontrivial(self):
    result = self.whop.getOptionsForField('testchild1')
    assertDiracSucceedsWith_equals(result, ['testoption', 'other_option', 'lastoneIpromise'], self)

  def test_changeandreturn_nontrivial(self):
    with self.assertRaises(KeyError) as ke:
      self.whop.changeAndReturn({})
    assertEqualsImproved(ke.exception.args[0], 'type', self)

  def test_changeandreturn_updatechecks(self):
    missing_values = {'testchild1': ['other_option', 'lastoneIpromise']}
    for key, list_of_subelems in missing_values.items():
      for subelem in list_of_subelems:
        self.whop.whizardxml.find('%s/%s' % (key, subelem)).attrib['type'] = 'default'
    result = self.whop.changeAndReturn({'testchild1': {'lastoneIpromise': 242}})
    assertDiracSucceeds(result, self)
    root = result['Value']
    assertEqualsImproved(root.find('testchild1/lastoneIpromise').attrib['value'], 242, self)

  def test_getasdict_nontrivial(self):
    missing_values = {'testchild1': ['other_option', 'lastoneIpromise'], 'dontforgetme': [],
                       'onlychild': []}
    root = self.whop.whizardxml
    for key, list_of_subelems in missing_values.items():
      for subelem in list_of_subelems:
        root.find('%s/%s' % (key, subelem)).attrib['value'] = 'default'
    result = self.whop.getAsDict()
    assertDiracSucceedsWith_equals(result, {'dontforgetme': {}, 'onlychild': {},
                                              'testchild1': {'lastoneIpromise': 'default',
                                                              'other_option': 'default',
                                                              'testoption': '123'}}, self)



"""Basic setup file to allow editable pip installs while using the declarative configuration (setup.cfg)."""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from setuptools import setup

# This is required
setup()



"""Module to encode and decode ILD filename meta information."""
from __future__ import absolute_import
import os
import re
import copy


class FilenameEncoder(object):
  """A utility class to decode a output filename from an input file name according to the file name convension used by ILD.

  Once rules are defined, output file, directory and meta values can be generated base on the `dict` object. See __main__ attached below.

  Examples to use this class will be found in ILCDIRAC/Core/Utilities/tests/Test_FilenameEncoder.py

  Following keys are used for ILDProduction
  [meta] is meta key defined for corresponding directory::

      %s: ILDConfig for simulation
      %r: ILDConfig for Marlin
      %m: Detector model
      %E: Energy-Machine
      %I: GenProcessID
      %P: ProcessName
      %C: Event Class
      %G: Generator program
      %e: electron polarization or type of photon beam
      %p: positron polarization or type of photon beam
      %d: Data type (gen, sim, rec, dst, dstm, .. )
      %t: Production ID
      %T: Directory name for Production ID
      %n: Generator file number ( could be [0-9]+_[0-9]+, when splitted )
      %j: Job number
      %J: Sub directory ( Job number/1000.  Namely 000, 001, 002, ... )
      %F: File type
      %B: Base directory
      %D: Upper case Data type. Used for meta value
      %w: Energy. for meta value
      %o: Machine parameter. such as TDR_ws for meta value

  :author: Akiya Miyamoto
  :date: 18 October 2016
  """

  def __init__(self):
    self.rules = {}
    self.rules["gen"] = {}
    self.rules["gen"]["file"] = "E%E.P%P.G%G.e%e.p%p.I%I.n%n.d_%d_%t_%j.%F"
    self.rules["gen"]["dir"] = "%B/%d/%E/%C/%T/%J"
    self.rules["gen"]["meta"] = {"%B/%d": {"Datatype": "%D"},
                                 "%B/%d/%E": {"Energy": "%w", "MachineParams": "%o"},
                                 "%B/%d/%E/%C": {"EventClass": "%C"},
                                 "%B/%d/%E/%C/%T": {"ProdID": "%t"},
                                 "%B/%d/%E/%C/%T/%J": {"kJobNumber": "%J"}}

    self.rules["sim"] = {}
    self.rules["sim"]["file"] = "s%s.m%m.E%E.I%I.P%P.e%e.p%p.n%n.d_%d_%t_%j.slcio"
    self.rules["sim"]["dir"] = "%B/%d/%E/%C/%m/%s/%T/%J"
    self.rules["sim"]["meta"] = {"%B/%d": {"Datatype": "%D"},
                                 "%B/%d/%E": {"Energy": "%w", "MachineParams": "%o"},
                                 "%B/%d/%E/%C": {"EventClass": "%C"},
                                 "%B/%d/%E/%C/%m": {"DetectorModel": "%m"},
                                 "%B/%d/%E/%C/%m/%s": {"ILDConfig": "%s"},
                                 "%B/%d/%E/%C/%m/%s/%T": {"ProdID": "%t"},
                                 "%B/%d/%E/%C/%m/%s/%T/%J": {"kJobNumber": "%J"}}

    self.rules["rec"] = {}
    self.rules["rec"]["file"] = "r%r.s%s.m%m.E%E.I%I.P%P.e%e.p%p.n%n.d_%d_%t_%j.slcio"
    self.rules["rec"]["dir"] = "%B/%d/%E/%C/%m/%r/%T/%J"
    self.rules["rec"]["meta"] = {"%B/%d": {"Datatype": "%D"},
                                 "%B/%d/%E": {"Energy": "%w", "MachineParams": "%o"},
                                 "%B/%d/%E/%C": {"EventClass": "%C"},
                                 "%B/%d/%E/%C/%m": {"DetectorModel": "%m"},
                                 "%B/%d/%E/%C/%m/%r": {"ILDConfig": "%r"},
                                 "%B/%d/%E/%C/%m/%r/%T": {"ProdID": "%t"},
                                 "%B/%d/%E/%C/%m/%r/%T/%J": {"kJobNumber": "%J"}}

    self.rules["dst"] = {}
    self.rules["dst"]["file"] = "r%r.s%s.m%m.E%E.I%I.P%P.e%e.p%p.n%n.d_%d_%t_%j.slcio"
    self.rules["dst"]["dir"] = self.rules["rec"]["dir"]
    self.rules["dst"]["meta"] = self.rules["rec"]["meta"]

# =====================================================
  def __del__(self):
    self.rules.clear()

# =====================================================
  def getARule(self, datatype, category=""):
    """returns a rule (rules) defined.

    :param str datatype: either "sim", "rec", "dst"
    :param str category: either "file", "dir", "meta"

    :returns: dict, except category="file" or "dir" is defined.
    """

    if category != "":
      return self.rules[datatype][category]

    return self.rules[datatype]

# =====================================================
  def defineRules(self, rule, datatype="", category=""):
    """Save rules for various name generation.

    :param dict rule: Rule(s) to make file names, directory name, or directory meta values
                       Example will be found in c'tor statements
    :param str datatype: data tytpe to which the defined rule is applied. For example, "sim", "rec", or "dst"
    :param str category: a kind of rule is whether it is for file name("file"), directory name("dir"),
       or directory meta values ("meta")
    """
    if datatype == "":
      self.rules = copy.deepcopy(rule)
    elif category == "":
      self.rules.update(rule)
    else:
      self.rules[datatype][category] = rule

# =====================================================
  def convert(self, datatype, category, values):
    """Calls file name, directory converter, or meta value maker depending on the input arguments and returns filename/directory name as a string or directory meta key and value as a dict object.

    :param str datatype: datatype defined by rules, for example sim, rec, dst, ...
    :param str category: Type of output converted.  file, dir (directory) or meta ( meta values )
    :param dict  values: Dictionary object for key word replacement

    :returns: file name or directory name as a string or a directory meta value if category is "meta"
    """

    if category == "meta":
      return makeDirMetaData(self.rules[datatype]["meta"], values)

    return makeFilename(self.rules[datatype][category], values)

# =====================================================


def decodeFilename(fullpath, separator="."):
  """Decode a file name to Key and Value according to the DBD file name convention. File name is splitted by ".", each item is decoded assuming it consits of 1 character of key followed by key value.  Exception seen in DBD generator files are also handled. Only basename of fullpath is used, even if direcories are included in fullpath.

  :param str fullpath: File name in fullpath is decoded.
  :returns: returns a dict object containing key and its value.
  """

  filename = os.path.basename(fullpath)
  if filename.count('_gen_'):
    # Special treatment for ILDDirac old files.
    ftemp = re.sub(r'([0-9a-zA-Z])_gen_([0-9]+_[0-9]+_[0-9]+).(stdhep|slcio)', r'\1.d_gen_\2.\3', filename)
  else:
    ftemp = re.sub(r'-(\d+).slcio', r'.j\1.slcio', filename)  # Special treatment for DBD sim files.
    # Special treatment for ILDDirac old files.
    ftemp = re.sub(r'([0-9a-zA-Z])_(sim|rec|dst)_(\d+_\d+).slcio', r'\1.d_\2_\3.slcio', ftemp)

  replaceList = [["Gwhizard-1.95", "Gwhizard-1_95"]]
  for replacement in replaceList:
    ftemp = ftemp.replace(replacement[0], replacement[1])

  filemeta = {}
  for token in ftemp.split(separator):
    conv = re.sub(r'^(\d)', r'n\1', token)

    conv = conv.replace("stdhep", "Fstdhep")
    conv = conv.replace("slcio", "Fslcio")
    key = conv[0:1]
    value = conv[1:]
    if key == "E":
      if value[0:1] == "0":
        value = value[1:]
    if key == "d":  # special treatment for old file names with prodID and job number
      if value[0:1] == "_":
        dsplit = value[1:].split('_')
        value = dsplit[0]
        filemeta['t'] = dsplit[1]
        if len(dsplit) == 4 and value == 'gen':
          filemeta['n'] = '%3.3d_%3.3d' % (int(dsplit[2]), int(dsplit[3]))
        else:
          filemeta['j'] = dsplit[2]

    filemeta[key] = value

  return filemeta

# =================================================


def makeFilename(fileformat, filemeta, preonly=True):
  """Make a filename, namely, Replace fileformat according to the filemeta.

  Filemeta is a `dict` objects, each entry being key and value. As a default, key is one character and "%[key]" in fileformat
  is replaced by "value".  If preonly is "False", "%[key]%" is replaced by value. In this case, [key] can be more than one
  character. Filename could be any string, like a fullpath.

  :param str fileformat: A string consisting of keys
  :param dict filemeta:  A dict object of keys and values

  :returns: filename
  :rtype: str
  """
  filename = fileformat
  for key, value in filemeta.items():
    target = "%" + key
    if not preonly:
      target = "%" + key + "%"

    filename = filename.replace(target, value)

  return filename

# =================================================


def makeDirMetaData(metaformat, items):
  """Returns a `dict` object which should be used for directory meta key and value definition.

  "%[key]" strings in a dict object, metaformat, is replaced according to items and a resultant `dict` object is returned.

  :param dict metaformat: Dictionary object defining the format. The key's and value's of
       metaformat should be string containing "%[key]"
  :param dict items: "%[key]"s in metaformat are replaced by a matching key-value in items

  :returns: Dictionary of directories and their metadata keys and values
  :rtype: dict
  """
  meta = {}
#   pprint.pprint(metaformat)

  for key, value in metaformat.items():
    newkey = key
    newvalue = value
#       print newkey
    for kitem, kvalue in items.items():
      newkey = newkey.replace("%" + kitem, kvalue)

    itemmeta = {}
    for varkey, varvalue in newvalue.items():
      vnew = varkey
      vval = varvalue
      for kitem, kvalue in items.items():
        vnew = vnew.replace("%" + kitem, kvalue)
        vval = vval.replace("%" + kitem, kvalue)
        itemmeta[vnew] = vval

    meta[newkey] = itemmeta

  return meta
